@@ -37,14 +37,14 @@ BUILD_DIR = "build-ci"
 NIX_TARGET = os.environ.get("NIX_TARGET", "")
 CMAKE_BUILD_TYPE = os.environ.get("CMAKE_BUILD_TYPE", "Release")
 RUN_TESTS = os.environ.get("RUN_TESTS", "1")
-RUN_MEMLEAK_TEST = os.environ.get("RUN_MEMLEAK_TEST", "0")
+RUN_AOT_TESTS = os.environ.get("RUN_AOT_TESTS", "0")
 CC = os.environ.get("CC", "cc")
 CXX = os.environ.get("CXX", "c++")
-GTEST_COLOR = os.environ.get("GTEST_COLOR", "auto")
 CI = os.environ.get("CI", "false")
-RUNTIME_TEST_COLOR = os.environ.get("RUNTIME_TEST_COLOR", "auto")
+NIX_TARGET_KERNEL = os.environ.get("NIX_TARGET_KERNEL", "")
 TOOLS_TEST_OLDVERSION = os.environ.get("TOOLS_TEST_OLDVERSION", "")
 TOOLS_TEST_DISABLE = os.environ.get("TOOLS_TEST_DISABLE", "")
+AOT_ALLOWLIST_FILE = os.environ.get("AOT_ALLOWLIST_FILE", "")
 
 
 class TestStatus(Enum):
@@ -124,6 +124,7 @@ def shell(
             # play nice with root or writing temporary files. So that
             # requires further investigation.
             "--preserve-env=PATH",
+            "--preserve-env=PYTHONPATH",
             # Also preserve any caller specified env vars
             f"--preserve-env={to_preserve}",
         ]
@@ -160,13 +161,12 @@ def configure():
         f"-DCMAKE_C_COMPILER={CC}",
         f"-DCMAKE_CXX_COMPILER={CXX}",
         f"-DCMAKE_BUILD_TYPE={CMAKE_BUILD_TYPE}",
-        f"-DBUILD_ASAN={RUN_MEMLEAK_TEST}",
 
         # Static configs
         f"-DCMAKE_VERBOSE_MAKEFILE=1",
         f"-DBUILD_TESTING=1",
         f"-DENABLE_SKB_OUTPUT=1",
-        f"-DALLOW_UNSAFE_PROBE=0",
+        f"-DBUILD_ASAN=1",
     ]
     # fmt: on
 
@@ -219,6 +219,43 @@ def tests_finish(results: List[TestResult]):
         print(output.getvalue())
 
 
+def run_runtime_tests():
+    """Runs runtime tests, under a controlled kernel if requested"""
+    script = "./tests/runtime-tests.sh"
+
+    if NIX_TARGET_KERNEL:
+        # Bring kernel into nix store then grab the path
+        subprocess.run([nix(), "build", NIX_TARGET_KERNEL], check=True)
+        eval = subprocess.run(
+            [nix(), "eval", "--raw", NIX_TARGET_KERNEL],
+            check=True,
+            capture_output=True,
+            text=True,
+        )
+
+        # nf_tables and xfs are necessary for testing kernel modules BTF support
+        modules = ["kvm", "nf_tables", "xfs"]
+        modprobe = f"modprobe -d {eval.stdout} -a {' '.join(modules)}"
+
+        c = f"{modprobe} && {script}"
+        cmd = ["vmtest", "-k", f"{eval.stdout}/bzImage", c]
+    else:
+        cmd = [script]
+
+    shell(
+        cmd,
+        # Don't need root if running tests in a VM
+        as_root=not NIX_TARGET_KERNEL,
+        cwd=Path(BUILD_DIR),
+        env={
+            "CI": CI,
+            "RUNTIME_TEST_COLOR": "yes",
+            # Disable UI to make CI and manual runs behave identically
+            "VMTEST_NO_UI": "1",
+        },
+    )
+
+
 def test():
     """
     Run all requested tests
@@ -238,7 +275,7 @@ def test():
             lambda: shell(
                 ["./tests/bpftrace_test"],
                 cwd=Path(BUILD_DIR),
-                env={"GTEST_COLOR": GTEST_COLOR},
+                env={"GTEST_COLOR": "yes"},
             ),
         )
     )
@@ -246,15 +283,7 @@ def test():
         test_one(
             "runtime-tests.sh",
             lambda: truthy(RUN_TESTS),
-            lambda: shell(
-                ["./tests/runtime-tests.sh"],
-                as_root=True,
-                cwd=Path(BUILD_DIR),
-                env={
-                    "CI": CI,
-                    "RUNTIME_TEST_COLOR": RUNTIME_TEST_COLOR,
-                },
-            ),
+            run_runtime_tests,
         )
     )
     results.append(
@@ -276,10 +305,29 @@ def test():
     )
     results.append(
         test_one(
-            "memleak-tests.sh.sh",
-            lambda: truthy(RUN_MEMLEAK_TEST),
+            "runtime-tests.sh (AOT)",
+            lambda: truthy(RUN_AOT_TESTS),
             lambda: shell(
-                ["./tests/memleak-tests.sh"], as_root=True, cwd=Path(BUILD_DIR)
+                (
+                    [
+                        "./tests/runtime-tests.sh",
+                        "--run-aot-tests",
+                        "--filter",
+                        "aot.*",
+                    ]
+                    + [
+                        "--allowlist_file",
+                        f"{root()}/{AOT_ALLOWLIST_FILE}",
+                    ]
+                    if AOT_ALLOWLIST_FILE
+                    else []
+                ),
+                as_root=True,
+                cwd=Path(BUILD_DIR),
+                env={
+                    "CI": CI,
+                    "RUNTIME_TEST_COLOR": "yes",
+                },
             ),
         )
     )
