@@ -4,11 +4,13 @@
 #include <cstdlib>
 #include <cstring>
 #include <exception>
-#include <fcntl.h>
+#include <filesystem>
 #include <fstream>
+#include <optional>
+
+#include <fcntl.h>
 #include <gelf.h>
 #include <libelf.h>
-#include <optional>
 #include <sys/mman.h>
 #include <sys/stat.h>
 
@@ -18,8 +20,6 @@
 #include <cereal/types/unordered_map.hpp>
 #include <cereal/types/vector.hpp>
 
-#include "ast/elf_parser.h"
-#include "filesystem.h"
 #include "log.h"
 #include "utils.h"
 #include "version.h"
@@ -47,8 +47,7 @@ struct Header {
 static_assert(sizeof(Header) == 48);
 static_assert(sizeof(std::size_t) <= sizeof(uint64_t));
 
-namespace bpftrace {
-namespace aot {
+namespace bpftrace::aot {
 namespace {
 
 uint32_t rs_hash(std::string_view str)
@@ -128,7 +127,7 @@ std::optional<std::vector<uint8_t>> generate_btaot_section(
 
 // Clones the shim to final destination while also injecting
 // the custom .btaot section.
-int build_binary(const std_filesystem::path &shim,
+int build_binary(const std::filesystem::path &shim,
                  const std::string &out,
                  const std::vector<uint8_t> &section)
 {
@@ -174,7 +173,7 @@ int build_binary(const std_filesystem::path &shim,
   }
 
 out:
-  if (!std_filesystem::remove(AOT_SECDATA_TEMPFILE, ec) || ec) {
+  if (!std::filesystem::remove(AOT_SECDATA_TEMPFILE, ec) || ec) {
     ret = 1;
     LOG(ERROR) << "Failed to remove " << AOT_SECDATA_TEMPFILE << ": " << ec;
   }
@@ -195,7 +194,7 @@ int generate(const RequiredResources &resources,
   // For development, we want to use the locally built AOT shim instead of a
   // "real" one that could be present elsewhere in $PATH. So we give precedence
   // to shim found "next" to the running binary.
-  auto rel = std_filesystem::path("aot") / AOT_SHIM_NAME;
+  auto rel = std::filesystem::path("aot") / AOT_SHIM_NAME;
   auto local = find_near_self(rel.native());
   auto path = find_in_path(AOT_SHIM_NAME);
   auto shim = local ? local : path;
@@ -227,7 +226,7 @@ int load(BPFtrace &bpftrace, const std::string &in)
   }
 
   std::error_code ec;
-  std_filesystem::path in_path{ in };
+  std::filesystem::path in_path{ in };
   std::uintmax_t in_file_size = std::filesystem::file_size(in_path, ec);
 
   if (in_file_size == static_cast<std::uintmax_t>(-1)) {
@@ -236,12 +235,12 @@ int load(BPFtrace &bpftrace, const std::string &in)
   }
 
   // Find .btaot section
-  Elf *elf = NULL;
-  Elf_Scn *scn = NULL;
+  Elf *elf = nullptr;
+  Elf_Scn *scn = nullptr;
   GElf_Shdr shdr;
-  char *secname = NULL;
-  Elf_Data *data = NULL;
-  uint8_t *btaot_section = NULL;
+  char *secname = nullptr;
+  Elf_Data *data = nullptr;
+  uint8_t *btaot_section = nullptr;
   const Header *hdr;
 
   if (elf_version(EV_CURRENT) == EV_NONE) {
@@ -250,7 +249,7 @@ int load(BPFtrace &bpftrace, const std::string &in)
     goto out;
   }
 
-  elf = elf_begin(infd, ELF_C_READ, NULL);
+  elf = elf_begin(infd, ELF_C_READ, nullptr);
   if (!elf) {
     LOG(ERROR) << "Cannot read ELF file: " << elf_errmsg(-1);
     err = 1;
@@ -285,7 +284,7 @@ int load(BPFtrace &bpftrace, const std::string &in)
     }
 
     if (std::string_view(secname) == AOT_ELF_SECTION) {
-      data = elf_getdata(scn, 0);
+      data = elf_getdata(scn, nullptr);
       if (!data) {
         LOG(ERROR) << "Failed to get BTAOT ELF section(" << i
                    << ") data: " << elf_errmsg(-1);
@@ -340,9 +339,8 @@ int load(BPFtrace &bpftrace, const std::string &in)
   if (err)
     goto out;
 
-  bpftrace.bytecode_ = elf::parseBpfBytecodeFromElfObject(btaot_section +
-                                                              hdr->elf_off,
-                                                          hdr->elf_len);
+  bpftrace.bytecode_ = BpfBytecode{ std::span<uint8_t>{
+      btaot_section + hdr->elf_off, hdr->elf_len } };
   if (err)
     goto out;
 
@@ -354,5 +352,4 @@ out:
   return err;
 }
 
-} // namespace aot
-} // namespace bpftrace
+} // namespace bpftrace::aot
