@@ -6,8 +6,7 @@
 // SP + 8 points to the first argument that is passed on the stack
 #define ARG0_STACK 8
 
-namespace bpftrace {
-namespace arch {
+namespace bpftrace::arch {
 
 // clang-format off
 static std::array<std::string, 27> registers = {
@@ -103,5 +102,4 @@ int get_kernel_ptr_width()
   return 64;
 }
 
-} // namespace arch
-} // namespace bpftrace
+} // namespace bpftrace::arch
