@@ -1,31 +1,40 @@
 #include "dibuilderbpf.h"
 
+#include <string_view>
+
+#include <llvm/IR/Function.h>
+
+#include "libbpf/bpf.h"
 #include "log.h"
 #include "struct.h"
 #include "utils.h"
 
-#include <llvm/IR/Function.h>
-
-namespace bpftrace {
-namespace ast {
+namespace bpftrace::ast {
 
 DIBuilderBPF::DIBuilderBPF(Module &module) : DIBuilder(module)
 {
   file = createFile("bpftrace.bpf.o", ".");
 }
 
-void DIBuilderBPF::createFunctionDebugInfo(Function &func)
+void DIBuilderBPF::createFunctionDebugInfo(llvm::Function &func,
+                                           const SizedType &ret_type,
+                                           const Struct &args,
+                                           bool is_declaration)
 {
-  // BPF probe function has:
-  // - int return type
-  // - single parameter (ctx) of a pointer type
-  SmallVector<Metadata *, 2> types = { getInt64Ty(), getInt8PtrTy() };
+  // Return type should be at index 0
+  SmallVector<Metadata *> types;
+  types.reserve(args.fields.size() + 1);
+  types.push_back(GetType(ret_type, false));
+  for (auto &arg : args.fields)
+    types.push_back(GetType(arg.type, false));
 
   DISubroutineType *ditype = createSubroutineType(getOrCreateTypeArray(types));
 
   std::string sanitised_name = sanitise_bpf_program_name(func.getName().str());
 
-  DISubprogram::DISPFlags flags = DISubprogram::SPFlagDefinition;
+  DISubprogram::DISPFlags flags = DISubprogram::SPFlagZero;
+  if (!is_declaration)
+    flags |= DISubprogram::SPFlagDefinition;
   if (func.isLocalLinkage(func.getLinkage()))
     flags |= DISubprogram::DISPFlags::SPFlagLocalToUnit;
 
@@ -38,12 +47,53 @@ void DIBuilderBPF::createFunctionDebugInfo(Function &func)
                                          0,
                                          DINode::FlagPrototyped,
                                          flags);
+#if LLVM_VERSION_MAJOR < 17
+  // There's a bug in LLVM <17 in DIBuilder::createFunction when called for a
+  // function declaration. It creates an empty temporary MDTuple for
+  // RetainedNodes inside DISubprogram which is, in addition, never freed.
+  //
+  // We generate function declaration debug info for kfuncs so this causes
+  // two issues when kfuncs are used on LLVM <17:
+  //
+  // 1. The generated LLVM IR is invalid and its verification will fail.
+  // 2. There is a memory leak introduced by the above createFunction call.
+  //
+  // To fix both problems, delete the temporary MDTuple here.
+  //
+  // Note that the issue was fixed in LLVM 17 by
+  //
+  //  https://github.com/llvm/llvm-project/commit/ed506dd6cecd9653cf9202bfe195891a33482852
+  //
+  // which removes the creation of the temporary MDTuple.
+  //
+  if (is_declaration) {
+    llvm::MDNode::deleteTemporary(subprog->getRetainedNodes().get());
+  }
+#endif
 
-  createParameterVariable(subprog, "ctx", 1, file, 0, (DIType *)types[1], true);
+  for (size_t i = 0; i < args.fields.size(); i++) {
+    createParameterVariable(subprog,
+                            args.fields.at(i).name,
+                            i + 1,
+                            file,
+                            0,
+                            static_cast<DIType *>(types[i + 1]),
+                            true);
+  }
 
   func.setSubprogram(subprog);
 }
 
+void DIBuilderBPF::createProbeDebugInfo(llvm::Function &probe_func)
+{
+  // BPF probe function has:
+  // - int return type
+  // - single parameter (ctx) of a pointer type
+  Struct args;
+  args.AddField("ctx", CreatePointer(CreateInt8()));
+  createFunctionDebugInfo(probe_func, CreateInt64(), args);
+}
+
 DIType *DIBuilderBPF::getInt8Ty()
 {
   if (!types_.int8)
@@ -123,12 +173,14 @@ DIType *DIBuilderBPF::CreateTupleType(const SizedType &stype)
   return result;
 }
 
-DIType *DIBuilderBPF::CreateMinMaxType(const SizedType &stype)
+DIType *DIBuilderBPF::CreateMapStructType(const SizedType &stype)
 {
-  assert(stype.IsMinTy() || stype.IsMaxTy());
+  assert(stype.IsMinTy() || stype.IsMaxTy() || stype.IsAvgTy() ||
+         stype.IsStatsTy());
 
-  // The first field is the value
-  // The second field is the "value is set" flag
+  // For Min/Max, the first field is the value and the second field is the
+  // "value is set" flag. For Avg/Stats, the first field is the total and the
+  // second field is the count.
   SmallVector<Metadata *, 2> fields = { createMemberType(file,
                                                          "",
                                                          file,
@@ -160,9 +212,51 @@ DIType *DIBuilderBPF::CreateMinMaxType(const SizedType &stype)
   return result;
 }
 
-DIType *DIBuilderBPF::GetType(const SizedType &stype)
+DIType *DIBuilderBPF::CreateByteArrayType(uint64_t num_bytes)
 {
-  if (stype.IsByteArray() || stype.IsRecordTy()) {
+  auto subrange = getOrCreateSubrange(0, num_bytes);
+  return createArrayType(
+      num_bytes * 8, 0, getInt8Ty(), getOrCreateArray({ subrange }));
+}
+
+/// Convert internal SizedType to a corresponding DIType type.
+///
+/// In codegen, some types are not converted into a directly corresponding
+/// LLVM type but instead into a type which is easy to work with in BPF
+/// programs (see IRBuilderBPF::GetType for details).
+///
+/// We do the same here for debug types and, similarly to IRBuilderBPF::GetType,
+/// allow to emit directly corresponding types by setting `emit_codegen_types`
+/// to false. This is necessary when emitting info for types whose BTF must
+/// exactly match the kernel BTF (e.g. kernel functions ("kfunc") prototypes).
+///
+/// Note: IRBuilderBPF::GetType doesn't implement creating actual struct types
+/// as it is not necessary for the current use-cases. For debug info types, this
+/// is not the case and we need to emit a struct type with at least the correct
+/// name and size (fields are not necessary).
+DIType *DIBuilderBPF::GetType(const SizedType &stype, bool emit_codegen_types)
+{
+  if (!emit_codegen_types && stype.IsRecordTy()) {
+    std::string name = stype.GetName();
+    static constexpr std::string_view struct_prefix = "struct ";
+    static constexpr std::string_view union_prefix = "union ";
+    if (name.find(struct_prefix) == 0)
+      name = name.substr(struct_prefix.length());
+    else if (name.find(union_prefix) == 0)
+      name = name.substr(union_prefix.length());
+
+    return createStructType(file,
+                            name,
+                            file,
+                            0,
+                            stype.GetSize() * 8,
+                            0,
+                            DINode::FlagZero,
+                            nullptr,
+                            getOrCreateArray({}));
+  }
+
+  if (stype.IsByteArray() || stype.IsRecordTy() || stype.IsStack()) {
     auto subrange = getOrCreateSubrange(0, stype.GetSize());
     return createArrayType(
         stype.GetSize() * 8, 0, getInt8Ty(), getOrCreateArray({ subrange }));
@@ -179,11 +273,15 @@ DIType *DIBuilderBPF::GetType(const SizedType &stype)
   if (stype.IsTupleTy())
     return CreateTupleType(stype);
 
-  if (stype.IsMinTy() || stype.IsMaxTy())
-    return CreateMinMaxType(stype);
+  if (stype.IsMinTy() || stype.IsMaxTy() || stype.IsAvgTy() ||
+      stype.IsStatsTy())
+    return CreateMapStructType(stype);
 
   if (stype.IsPtrTy())
-    return getInt64Ty();
+    return emit_codegen_types ? getInt64Ty()
+                              : createPointerType(GetType(*stype.GetPointeeTy(),
+                                                          emit_codegen_types),
+                                                  64);
 
   // Integer types and builtin types represented by integers
   switch (stype.GetSize()) {
@@ -203,29 +301,26 @@ DIType *DIBuilderBPF::GetType(const SizedType &stype)
   }
 }
 
-DIType *DIBuilderBPF::GetMapKeyType(const MapKey &key,
-                                    const SizedType &value_type)
+DIType *DIBuilderBPF::GetMapKeyType(const SizedType &key_type,
+                                    const SizedType &value_type,
+                                    libbpf::bpf_map_type map_type)
 {
   // No-key maps use '0' as the key.
-  // No-key count() maps use BPF_MAP_TYPE_PERCPU_ARRAY which needs 4-byte key.
-  if (key.args_.size() == 0)
-    return value_type.IsCountTy() ? getInt32Ty() : getInt64Ty();
+  // - BPF requires 4-byte keys for array maps
+  // - bpftrace uses 8 bytes for the implicit '0' key in hash maps
+  if (key_type.IsNoneTy())
+    return (map_type == libbpf::BPF_MAP_TYPE_PERCPU_ARRAY ||
+            map_type == libbpf::BPF_MAP_TYPE_ARRAY)
+               ? getInt32Ty()
+               : getInt64Ty();
 
   // Some map types need an extra 8-byte key.
-  uint64_t extra_arg_size = 0;
-  if (value_type.IsHistTy() || value_type.IsLhistTy() || value_type.IsAvgTy() ||
-      value_type.IsStatsTy())
-    extra_arg_size = 8;
+  if (value_type.IsHistTy() || value_type.IsLhistTy()) {
+    uint64_t size = key_type.GetSize() + 8;
+    return CreateByteArrayType(size);
+  }
 
-  // Single map key -> use the appropriate type.
-  if (key.args_.size() == 1 && extra_arg_size == 0)
-    return GetType(key.args_[0]);
-
-  // Multi map key -> use byte array.
-  uint64_t size = key.size() + extra_arg_size;
-  auto subrange = getOrCreateSubrange(0, size);
-  return createArrayType(
-      size * 8, 0, getInt8Ty(), getOrCreateArray({ subrange }));
+  return GetType(key_type);
 }
 
 DIType *DIBuilderBPF::GetMapFieldInt(int value)
@@ -250,7 +345,7 @@ DIGlobalVariableExpression *DIBuilderBPF::createMapEntry(
     const std::string &name,
     libbpf::bpf_map_type map_type,
     uint64_t max_entries,
-    const MapKey &key,
+    DIType *key_type,
     const SizedType &value_type)
 {
   SmallVector<Metadata *, 4> fields = {
@@ -260,8 +355,8 @@ DIGlobalVariableExpression *DIBuilderBPF::createMapEntry(
 
   uint64_t size = 128;
   if (!value_type.IsNoneTy()) {
-    fields.push_back(createPointerMemberType(
-        "key", size, createPointerType(GetMapKeyType(key, value_type), 64)));
+    fields.push_back(
+        createPointerMemberType("key", size, createPointerType(key_type, 64)));
     fields.push_back(createPointerMemberType(
         "value", size + 64, createPointerType(GetType(value_type), 64)));
     size += 128;
@@ -280,5 +375,12 @@ DIGlobalVariableExpression *DIBuilderBPF::createMapEntry(
       file, name, "global", file, 0, map_entry_type, false);
 }
 
-} // namespace ast
-} // namespace bpftrace
+DIGlobalVariableExpression *DIBuilderBPF::createGlobalVariable(
+    std::string_view name,
+    const SizedType &stype)
+{
+  return createGlobalVariableExpression(
+      file, name, "global", file, 0, GetType(stype, false), false);
+}
+
+} // namespace bpftrace::ast
