@@ -8,16 +8,7 @@
 #include "log.h"
 #include "types.h"
 
-namespace bpftrace {
-namespace ast {
-
-template <class... Ts>
-struct overloaded : Ts... {
-  using Ts::operator()...;
-};
-// explicit deduction guide (not needed as of C++20)
-template <class... Ts>
-overloaded(Ts...) -> overloaded<Ts...>;
+namespace bpftrace::ast {
 
 void ConfigAnalyser::log_type_error(SizedType &type,
                                     Type expected_type,
@@ -28,8 +19,8 @@ void ConfigAnalyser::log_type_error(SizedType &type,
       << ". Type: " << type.GetTy() << ". Expected Type: " << expected_type;
 }
 
-void ConfigAnalyser::set_uint64_config(AssignConfigVarStatement &assignment,
-                                       ConfigKeyInt key)
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                ConfigKeyInt key)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsIntegerTy()) {
@@ -40,8 +31,8 @@ void ConfigAnalyser::set_uint64_config(AssignConfigVarStatement &assignment,
   config_setter_.set(key, dynamic_cast<Integer *>(assignment.expr)->n);
 }
 
-void ConfigAnalyser::set_bool_config(AssignConfigVarStatement &assignment,
-                                     ConfigKeyBool key)
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                ConfigKeyBool key)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsIntegerTy()) {
@@ -60,8 +51,8 @@ void ConfigAnalyser::set_bool_config(AssignConfigVarStatement &assignment,
   }
 }
 
-void ConfigAnalyser::set_string_config(AssignConfigVarStatement &assignment,
-                                       ConfigKeyString key)
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                [[maybe_unused]] ConfigKeyString key)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsStringTy()) {
@@ -72,7 +63,8 @@ void ConfigAnalyser::set_string_config(AssignConfigVarStatement &assignment,
   config_setter_.set(key, dynamic_cast<String *>(assignment.expr)->str);
 }
 
-void ConfigAnalyser::set_stack_mode_config(AssignConfigVarStatement &assignment)
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                [[maybe_unused]] ConfigKeyStackMode)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsStackModeTy()) {
@@ -83,8 +75,9 @@ void ConfigAnalyser::set_stack_mode_config(AssignConfigVarStatement &assignment)
   config_setter_.set(assignTy.stack_type.mode);
 }
 
-void ConfigAnalyser::set_user_symbol_cache_type_config(
-    AssignConfigVarStatement &assignment)
+void ConfigAnalyser::set_config(
+    AssignConfigVarStatement &assignment,
+    [[maybe_unused]] ConfigKeyUserSymbolCacheType key)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsStringTy()) {
@@ -97,8 +90,22 @@ void ConfigAnalyser::set_user_symbol_cache_type_config(
     LOG(ERROR, assignment.expr->loc, err_);
 }
 
-void ConfigAnalyser::set_missing_probes_config(
-    AssignConfigVarStatement &assignment)
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                [[maybe_unused]] ConfigKeySymbolSource key)
+{
+  auto &assignTy = assignment.expr->type;
+  if (!assignTy.IsStringTy()) {
+    log_type_error(assignTy, Type::string, assignment);
+    return;
+  }
+
+  auto val = dynamic_cast<String *>(assignment.expr)->str;
+  if (!config_setter_.set_symbol_source_config(val))
+    LOG(ERROR, assignment.expr->loc, err_);
+}
+
+void ConfigAnalyser::set_config(AssignConfigVarStatement &assignment,
+                                [[maybe_unused]] ConfigKeyMissingProbes key)
 {
   auto &assignTy = assignment.expr->type;
   if (!assignTy.IsStringTy()) {
@@ -135,7 +142,7 @@ void ConfigAnalyser::visit(StackMode &mode)
 
 void ConfigAnalyser::visit(AssignConfigVarStatement &assignment)
 {
-  Visitor::visit(assignment);
+  Visitor<ConfigAnalyser>::visit(assignment);
   std::string &raw_ident = assignment.config_var;
 
   std::string err_msg;
@@ -149,32 +156,18 @@ void ConfigAnalyser::visit(AssignConfigVarStatement &assignment)
 
   if (!assignment.expr->is_literal) {
     LOG(ERROR, assignment.loc, err_)
-        << "Assignemnt for " << assignment.config_var << " must be literal.";
+        << "Assignment for " << assignment.config_var << " must be literal.";
     return;
   }
 
   auto configKey = maybeConfigKey.value();
 
-  std::visit(
-      overloaded{
-          [&, this](ConfigKeyBool key) { set_bool_config(assignment, key); },
-          [&, this](ConfigKeyInt key) { set_uint64_config(assignment, key); },
-          [&, this](ConfigKeyString key) {
-            set_string_config(assignment, key);
-          },
-          [&, this](ConfigKeyStackMode) { set_stack_mode_config(assignment); },
-          [&, this](ConfigKeyUserSymbolCacheType) {
-            set_user_symbol_cache_type_config(assignment);
-          },
-          [&, this](ConfigKeyMissingProbes) {
-            set_missing_probes_config(assignment);
-          } },
-      configKey);
+  std::visit([&](auto key) { set_config(assignment, key); }, configKey);
 }
 
 bool ConfigAnalyser::analyse()
 {
-  Visit(*root_);
+  visit(ctx_.root);
   std::string errors = err_.str();
   if (!errors.empty()) {
     out_ << errors;
@@ -185,8 +178,8 @@ bool ConfigAnalyser::analyse()
 
 Pass CreateConfigPass()
 {
-  auto fn = [](Node &n, PassContext &ctx) {
-    auto configs = ConfigAnalyser(&n, ctx.b);
+  auto fn = [](PassContext &ctx) {
+    auto configs = ConfigAnalyser(ctx.ast_ctx, ctx.b);
     if (!configs.analyse())
       return PassResult::Error("Config");
     return PassResult::Success();
@@ -195,5 +188,4 @@ Pass CreateConfigPass()
   return Pass("ConfigAnalyser", fn);
 };
 
-} // namespace ast
-} // namespace bpftrace
+} // namespace bpftrace::ast
