@@ -10,6 +10,7 @@
 #include "btf.h"
 #include "config.h"
 #include "types.h"
+#include "usdt.h"
 
 #include <bcc/libbpf.h>
 
@@ -21,12 +22,9 @@ std::string progtypeName(libbpf::bpf_prog_type t);
 
 class AttachedProbe {
 public:
+  AttachedProbe(Probe &probe, const BpfProgram &prog, BPFtrace &bpftrace);
   AttachedProbe(Probe &probe,
-                BpfProgram &&prog,
-                bool safe_mode,
-                BPFtrace &bpftrace);
-  AttachedProbe(Probe &probe,
-                BpfProgram &&prog,
+                const BpfProgram &prog,
                 int pid,
                 BPFtrace &bpftrace,
                 bool safe_mode = true);
@@ -41,12 +39,11 @@ public:
 private:
   std::string eventprefix() const;
   std::string eventname() const;
-  void resolve_offset_kprobe(bool safe_mode);
+  void resolve_offset_kprobe();
   bool resolve_offset_uprobe(bool safe_mode, bool has_multiple_aps);
-  void load_prog(BPFfeature &feature);
   void attach_multi_kprobe(void);
   void attach_multi_uprobe(int pid);
-  void attach_kprobe(bool safe_mode);
+  void attach_kprobe();
   void attach_uprobe(int pid, bool safe_mode);
 
   // Note: the following usdt attachment functions will only activate a
@@ -69,8 +66,8 @@ private:
   void attach_software();
   void attach_hardware();
   void attach_watchpoint(int pid, const std::string &mode);
-  void attach_kfunc(void);
-  int detach_kfunc(void);
+  void attach_fentry(void);
+  int detach_fentry(void);
   void attach_iter(void);
   int detach_iter(void);
   void attach_raw_tracepoint(void);
@@ -81,7 +78,6 @@ private:
   void cache_progfd(void);
 
   Probe &probe_;
-  BpfProgram prog_;
   std::vector<int> perf_event_fds_;
   bool close_progfd_ = true;
   int progfd_ = -1;
