@@ -1,26 +1,26 @@
 #include "bpffeature.h"
 
-#include <bcc/libbpf.h>
 #include <bpf/bpf.h>
 #include <bpf/btf.h>
 #include <bpf/libbpf.h>
 #include <cstddef>
+#include <cstdint>
 #include <cstdio>
 #include <fcntl.h>
 #include <fstream>
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "bpf_assembler.h"
 #include "btf.h"
 #include "debugfs.h"
+#include "dwarf_parser.h"
 #include "probe_matcher.h"
 #include "tracefs.h"
 #include "utils.h"
 
 namespace bpftrace {
 
-#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
-
 int BPFnofeature::parse(const char* str)
 {
   for (auto feat : split_string(str, ',')) {
@@ -59,7 +59,7 @@ static bool try_load_(const char* name,
       continue;
     }
 
-    LIBBPF_OPTS(bpf_prog_load_opts, opts);
+    BPFTRACE_LIBBPF_OPTS(bpf_prog_load_opts, opts);
     opts.log_buf = logbuf;
     opts.log_size = logbuf_size;
     opts.log_level = loglevel;
@@ -102,10 +102,7 @@ bool BPFfeature::try_load(enum libbpf::bpf_prog_type prog_type,
 
   std::optional<unsigned> btf_id;
   if (prog_type == libbpf::BPF_PROG_TYPE_TRACING && has_btf()) {
-    auto id_fd = btf_.get_btf_id_fd(name, "vmlinux");
-    btf_id = id_fd.first;
-    if (id_fd.second >= 0)
-      close(id_fd.second);
+    btf_id = btf_.get_btf_id(name, "vmlinux");
   }
 
   if (prog_type == libbpf::BPF_PROG_TYPE_TRACING) {
@@ -131,11 +128,11 @@ bool BPFfeature::try_load_btf(const void* btf_data, size_t btf_size)
 {
   constexpr int log_size = 4096;
   char log_buf[log_size] = {};
-  LIBBPF_OPTS(bpf_btf_load_opts,
-              btf_opts,
-              .log_buf = log_buf,
-              .log_level = 0,
-              .log_size = log_size, );
+  BPFTRACE_LIBBPF_OPTS(bpf_btf_load_opts,
+                       btf_opts,
+                       .log_buf = log_buf,
+                       .log_level = 0,
+                       .log_size = log_size, );
 
   int fd = bpf_btf_load(btf_data, btf_size, &btf_opts);
   if (fd >= 0) {
@@ -220,7 +217,7 @@ bool BPFfeature::detect_map(enum libbpf::bpf_map_type map_type)
       break;
   }
 
-  LIBBPF_OPTS(bpf_map_create_opts, opts);
+  BPFTRACE_LIBBPF_OPTS(bpf_map_create_opts, opts);
   opts.map_flags = flags;
   map_fd = bpf_map_create(static_cast<enum ::bpf_map_type>(map_type),
                           nullptr,
@@ -235,25 +232,7 @@ bool BPFfeature::detect_map(enum libbpf::bpf_map_type map_type)
   return map_fd >= 0;
 }
 
-bool BPFfeature::has_loop(void)
-{
-  if (has_loop_.has_value())
-    return *has_loop_;
-
-  struct bpf_insn insns[] = {
-    BPF_MOV64_IMM(BPF_REG_0, 0),
-    BPF_ALU64_IMM(BPF_ADD, BPF_REG_0, 1),
-    BPF_JMP_IMM(BPF_JLT, BPF_REG_0, 4, -2),
-    BPF_EXIT_INSN(),
-  };
-
-  has_loop_ = std::make_optional<bool>(
-      try_load(libbpf::BPF_PROG_TYPE_TRACEPOINT, insns, ARRAY_SIZE(insns)));
-
-  return has_loop();
-}
-
-bool BPFfeature::has_btf(void)
+bool BPFfeature::has_btf()
 {
   return btf_.has_data();
 }
@@ -279,7 +258,7 @@ bool BPFfeature::has_btf_func_global()
   return *has_btf_func_global_;
 }
 
-int BPFfeature::instruction_limit(void)
+int BPFfeature::instruction_limit()
 {
   if (insns_limit_.has_value())
     return *insns_limit_;
@@ -341,7 +320,7 @@ bool BPFfeature::has_map_batch()
   if (has_map_batch_.has_value())
     return *has_map_batch_;
 
-  LIBBPF_OPTS(bpf_map_create_opts, opts);
+  BPFTRACE_LIBBPF_OPTS(bpf_map_create_opts, opts);
   opts.map_flags = flags;
   map_fd = bpf_map_create(static_cast<enum ::bpf_map_type>(
                               libbpf::BPF_MAP_TYPE_HASH),
@@ -362,13 +341,13 @@ bool BPFfeature::has_map_batch()
   return *has_map_batch_;
 }
 
-bool BPFfeature::has_d_path(void)
+bool BPFfeature::has_d_path()
 {
   if (has_d_path_.has_value())
     return *has_d_path_;
 
   struct bpf_insn insns[] = {
-    BPF_LDX_MEM(BPF_W, BPF_REG_1, BPF_REG_1, 0),
+    BPF_LDX_MEM(BPF_DW, BPF_REG_1, BPF_REG_1, 0),
     BPF_MOV64_REG(BPF_REG_2, BPF_REG_10),
     BPF_ALU64_IMM(BPF_ADD, BPF_REG_2, -8),
     BPF_MOV64_IMM(BPF_REG_6, 0),
@@ -394,10 +373,10 @@ bool BPFfeature::has_uprobe_refcnt()
     return *has_uprobe_refcnt_;
 
   std::error_code ec;
-  std_filesystem::path path{
+  std::filesystem::path path{
     "/sys/bus/event_source/devices/uprobe/format/ref_ctr_offset"
   };
-  has_uprobe_refcnt_ = std_filesystem::exists(path, ec);
+  has_uprobe_refcnt_ = std::filesystem::exists(path, ec);
 
   return *has_uprobe_refcnt_;
 }
@@ -413,7 +392,7 @@ bool BPFfeature::has_kprobe_multi()
   }
 
   const char* sym = "ksys_read";
-  DECLARE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
+  BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
   int progfd, linkfd = -1;
 
   struct bpf_insn insns[] = {
@@ -424,7 +403,7 @@ bool BPFfeature::has_kprobe_multi()
   link_opts.kprobe_multi.syms = &sym;
   link_opts.kprobe_multi.cnt = 1;
 
-  LIBBPF_OPTS(bpf_prog_load_opts, load_opts);
+  BPFTRACE_LIBBPF_OPTS(bpf_prog_load_opts, load_opts);
   load_opts.expected_attach_type = static_cast<enum ::bpf_attach_type>(
       libbpf::BPF_TRACE_KPROBE_MULTI);
 
@@ -466,10 +445,11 @@ bool BPFfeature::has_uprobe_multi()
     return *has_uprobe_multi_;
   }
 
-  LIBBPF_OPTS(bpf_prog_load_opts,
-              load_opts,
-              .expected_attach_type = static_cast<enum ::bpf_attach_type>(
-                  libbpf::BPF_TRACE_UPROBE_MULTI), );
+  BPFTRACE_LIBBPF_OPTS(
+      bpf_prog_load_opts,
+      load_opts,
+      .expected_attach_type = static_cast<enum ::bpf_attach_type>(
+          libbpf::BPF_TRACE_UPROBE_MULTI), );
 
   int err = 0, progfd, linkfd = -1;
 
@@ -487,7 +467,7 @@ bool BPFfeature::has_uprobe_multi()
                          &load_opts);
 
   if (progfd >= 0) {
-    LIBBPF_OPTS(bpf_link_create_opts, link_opts);
+    BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
     const unsigned long offset = 0;
 
     link_opts.uprobe_multi.path = "/";
@@ -512,13 +492,13 @@ bool BPFfeature::has_uprobe_multi()
   }
 #else
   has_uprobe_multi_ = false;
-#endif // HAVE_LIBBPF_UPROBE_MULTI
-  return *has_uprobe_multi_;
+#endif                       // HAVE_LIBBPF_UPROBE_MULTI
+  return *has_uprobe_multi_; // NOLINT(bugprone-unchecked-optional-access)
 }
 
-bool BPFfeature::has_skb_output(void)
+bool BPFfeature::has_skb_output()
 {
-  if (!has_kfunc())
+  if (!has_fentry())
     return false;
 
   if (has_skb_output_.has_value())
@@ -526,7 +506,7 @@ bool BPFfeature::has_skb_output(void)
 
   int map_fd = 0;
 
-  LIBBPF_OPTS(bpf_map_create_opts, opts);
+  BPFTRACE_LIBBPF_OPTS(bpf_map_create_opts, opts);
   opts.map_flags = 0;
   map_fd = bpf_map_create(static_cast<enum ::bpf_map_type>(
                               libbpf::BPF_MAP_TYPE_PERF_EVENT_ARRAY),
@@ -540,7 +520,7 @@ bool BPFfeature::has_skb_output(void)
     return false;
 
   struct bpf_insn insns[] = {
-    BPF_LDX_MEM(BPF_W, BPF_REG_1, BPF_REG_1, 0),
+    BPF_LDX_MEM(BPF_DW, BPF_REG_1, BPF_REG_1, 0),
     BPF_LD_MAP_FD(BPF_REG_2, map_fd),
     BPF_MOV64_IMM(BPF_REG_3, 0),
     BPF_MOV64_REG(BPF_REG_4, BPF_REG_10),
@@ -564,93 +544,100 @@ bool BPFfeature::has_skb_output(void)
   return *has_skb_output_;
 }
 
-bool BPFfeature::has_raw_tp_special()
+static void tabulate(std::stringstream& buf,
+                     std::vector<std::pair<std::string, std::string>>& data)
 {
-  if (has_raw_tp_special_.has_value())
-    return *has_raw_tp_special_;
-
-  struct bpf_insn insns[] = { BPF_MOV64_IMM(BPF_REG_0, 0), BPF_EXIT_INSN() };
-  int fd;
-
-  // Check that we can both load BPF_PROG_TYPE_RAW_TRACEPOINT and that
-  // BPF_PROG_RUN is supported by the kernel
-  if (try_load(libbpf::BPF_PROG_TYPE_RAW_TRACEPOINT,
-               insns,
-               ARRAY_SIZE(insns),
-               nullptr,
-               std::nullopt,
-               &fd)) {
-    struct bpf_test_run_opts opts = {};
-    opts.sz = sizeof(opts);
-    has_raw_tp_special_ = !::bpf_prog_test_run_opts(fd, &opts);
-    close(fd);
-  } else
-    has_raw_tp_special_ = false;
-
-  return *has_raw_tp_special_;
+  size_t len = data.size();
+  constexpr int width = 35;
+  for (size_t i = 0; i < len; i += 2) {
+    buf << std::setw(width) << std::left
+        << "  " + data[i].first + ": " + data[i].second << std::setw(width);
+    if (i + 1 < len) {
+      buf << data[i + 1].first + ": " + data[i + 1].second << std::endl;
+    } else {
+      buf << std::endl;
+    }
+  }
 }
 
-std::string BPFfeature::report(void)
+std::string BPFfeature::report()
 {
   std::stringstream buf;
-  auto to_str = [](bool f) -> auto { return f ? "yes\n" : "no\n"; };
 
-  buf << "Kernel helpers" << std::endl
-      << "  probe_read: " << to_str(has_helper_probe_read())
-      << "  probe_read_str: " << to_str(has_helper_probe_read_str())
-      << "  probe_read_user: " << to_str(has_helper_probe_read_user())
-      << "  probe_read_user_str: " << to_str(has_helper_probe_read_user_str())
-      << "  probe_read_kernel: " << to_str(has_helper_probe_read_kernel())
-      << "  probe_read_kernel_str: "
-      << to_str(has_helper_probe_read_kernel_str())
-      << "  get_current_cgroup_id: "
-      << to_str(has_helper_get_current_cgroup_id())
-      << "  send_signal: " << to_str(has_helper_send_signal())
-      << "  override_return: " << to_str(has_helper_override_return())
-      << "  get_boot_ns: " << to_str(has_helper_ktime_get_boot_ns())
-      << "  dpath: " << to_str(has_d_path())
-      << "  skboutput: " << to_str(has_skb_output())
-      << "  get_tai_ns: " << to_str(has_helper_ktime_get_tai_ns())
-      << "  get_func_ip: " << to_str(has_helper_get_func_ip())
-      << "  jiffies64: " << to_str(has_helper_jiffies64())
-      << "  for_each_map_elem: " << to_str(has_helper_for_each_map_elem())
+  auto to_str = [](bool f) -> std::string { return f ? "yes" : "no"; };
 
-      << std::endl;
+  std::vector<std::pair<std::string, std::string>> helpers = {
+    { "probe_read", to_str(has_helper_probe_read()) },
+    { "probe_read_str", to_str(has_helper_probe_read_str()) },
+    { "probe_read_user", to_str(has_helper_probe_read_user()) },
+    { "probe_read_user_str", to_str(has_helper_probe_read_user_str()) },
+    { "probe_read_kernel", to_str(has_helper_probe_read_kernel()) },
+    { "probe_read_kernel_str", to_str(has_helper_probe_read_kernel_str()) },
+    { "get_current_cgroup_id", to_str(has_helper_get_current_cgroup_id()) },
+    { "send_signal", to_str(has_helper_send_signal()) },
+    { "override_return", to_str(has_helper_override_return()) },
+    { "get_boot_ns", to_str(has_helper_ktime_get_boot_ns()) },
+    { "dpath", to_str(has_d_path()) },
+    { "skboutput", to_str(has_skb_output()) },
+    { "get_tai_ns", to_str(has_helper_ktime_get_tai_ns()) },
+    { "get_func_ip", to_str(has_helper_get_func_ip()) },
+    { "jiffies64", to_str(has_helper_jiffies64()) },
+    { "for_each_map_elem", to_str(has_helper_for_each_map_elem()) },
+    { "get_ns_current_pid_tgid", to_str(has_helper_get_ns_current_pid_tgid()) },
+    { "lookup_percpu_elem", to_str(has_helper_map_lookup_percpu_elem()) },
+  };
 
-  buf << "Kernel features" << std::endl
-      << "  Instruction limit: " << instruction_limit() << std::endl
-      << "  Loop support: " << to_str(has_loop())
-      << "  btf: " << to_str(has_btf())
-      << "  module btf: " << to_str(has_module_btf())
-      << "  map batch: " << to_str(has_map_batch())
-      << "  uprobe refcount (depends on Build:bcc bpf_attach_uprobe refcount): "
-      << to_str(has_uprobe_refcnt()) << std::endl;
+  std::vector<std::pair<std::string, std::string>> features = {
+    { "Instruction limit", std::to_string(instruction_limit()) },
+    { "btf", to_str(has_btf()) },
+    { "module btf", to_str(has_module_btf()) },
+    { "Kernel DWARF", to_str(has_kernel_dwarf()) },
+    { "map batch", to_str(has_map_batch()) },
+    // Depends on BCC's bpf_attach_uprobe refcount feature
+    { "uprobe refcount", to_str(has_uprobe_refcnt()) }
+  };
 
-  buf << "Map types" << std::endl
-      << "  hash: " << to_str(has_map_hash())
-      << "  percpu hash: " << to_str(has_map_percpu_hash())
-      << "  array: " << to_str(has_map_array())
-      << "  percpu array: " << to_str(has_map_percpu_array())
-      << "  stack_trace: " << to_str(has_map_stack_trace())
-      << "  perf_event_array: " << to_str(has_map_perf_event_array())
-      << "  ringbuf: " << to_str(has_map_ringbuf()) << std::endl;
+  std::vector<std::pair<std::string, std::string>> map_types = {
+    { "hash", to_str(has_map_hash()) },
+    { "array", to_str(has_map_array()) },
+    { "percpu array", to_str(has_map_percpu_array()) },
+    { "stack_trace", to_str(has_map_stack_trace()) },
+    { "perf_event_array", to_str(has_map_perf_event_array()) },
+    { "ringbuf", to_str(has_map_ringbuf()) }
+  };
 
-  buf << "Probe types" << std::endl
-      << "  kprobe: " << to_str(has_prog_kprobe())
-      << "  tracepoint: " << to_str(has_prog_tracepoint())
-      << "  perf_event: " << to_str(has_prog_perf_event())
-      << "  kfunc: " << to_str(has_kfunc())
-      << "  kprobe_multi: " << to_str(has_kprobe_multi())
-      << "  uprobe_multi: " << to_str(has_uprobe_multi())
-      << "  raw_tp_special: " << to_str(has_raw_tp_special())
-      << "  iter: " << to_str(has_iter("task")) << std::endl;
+  std::vector<std::pair<std::string, std::string>> probe_types = {
+    { "kprobe", to_str(has_prog_kprobe()) },
+    { "tracepoint", to_str(has_prog_tracepoint()) },
+    { "perf_event", to_str(has_prog_perf_event()) },
+    { "fentry", to_str(has_fentry()) },
+    { "kprobe_multi", to_str(has_kprobe_multi()) },
+    { "uprobe_multi", to_str(has_uprobe_multi()) },
+    { "iter", to_str(has_iter("task")) }
+  };
+
+  buf << "Kernel helpers" << std::endl;
+  tabulate(buf, helpers);
+  buf << std::endl;
+
+  buf << "Kernel features" << std::endl;
+  tabulate(buf, features);
+  buf << std::endl;
+
+  buf << "Map types" << std::endl;
+  tabulate(buf, map_types);
+  buf << std::endl;
+
+  buf << "Probe types" << std::endl;
+  tabulate(buf, probe_types);
+  buf << std::endl;
 
   return buf.str();
 }
 
-bool BPFfeature::has_prog_kfunc()
+bool BPFfeature::has_prog_fentry()
 {
-  if (!has_prog_kfunc_.has_value()) {
+  if (!has_prog_fentry_.has_value()) {
     int progfd;
     if (!detect_prog_type(libbpf::BPF_PROG_TYPE_TRACING,
                           "sched_fork",
@@ -662,17 +649,17 @@ bool BPFfeature::has_prog_kfunc()
     if (tracing_fd < 0)
       goto out_false;
     close(tracing_fd);
-    has_prog_kfunc_ = std::make_optional<bool>(true);
+    has_prog_fentry_ = std::make_optional<bool>(true);
   }
-  return *(has_prog_kfunc_);
+  return *(has_prog_fentry_);
 out_false:
-  has_prog_kfunc_ = std::make_optional<bool>(false);
-  return *(has_prog_kfunc_);
+  has_prog_fentry_ = std::make_optional<bool>(false);
+  return *(has_prog_fentry_);
 }
 
-bool BPFfeature::has_kfunc()
+bool BPFfeature::has_fentry()
 {
-  return has_prog_kfunc() && btf_.has_data();
+  return has_prog_fentry() && btf_.has_data();
 }
 
 bool BPFfeature::has_module_btf()
@@ -682,7 +669,7 @@ bool BPFfeature::has_module_btf()
 
   char name[64];
   struct bpf_btf_info info = {};
-  info.name = (__u64)name;
+  info.name = reinterpret_cast<uintptr_t>(name);
   info.name_len = sizeof(name);
   __u32 id = 0, info_len = sizeof(info);
   int err = 0, fd = -1;
@@ -716,4 +703,40 @@ bool BPFfeature::has_iter(std::string name)
                           libbpf::BPF_TRACE_ITER);
 }
 
+bool BPFfeature::has_kernel_dwarf()
+{
+#ifndef HAVE_LIBLLDB
+  return false;
+#endif
+
+  auto vmlinux = find_vmlinux();
+  if (!vmlinux.has_value())
+    return false;
+
+  // WARNING: we are not passing a pointer to BPFtrace, so we can only use:
+  // * Dwarf::has_debug_info
+  // * Dwarf::get_function_locations
+  // * Dwarf::get_function_params
+  // Otherwise, Dwarf will try to use the BPFtrace pointer and will segfault.
+  auto dwarf = Dwarf::GetFromBinary(nullptr, vmlinux.value());
+  if (!dwarf)
+    return false;
+
+  return dwarf->has_debug_info();
+}
+
+bool BPFfeature::has_kernel_func(Kfunc kfunc)
+{
+  if (!has_btf())
+    return false;
+
+  auto find_kfunc = available_kernel_funcs_.find(kfunc);
+  if (find_kfunc != available_kernel_funcs_.end())
+    return find_kfunc->second;
+
+  bool result = btf_.get_btf_id(kfunc_name(kfunc), "") >= 0;
+  available_kernel_funcs_.emplace(kfunc, result);
+  return result;
+}
+
 } // namespace bpftrace
