@@ -1,6 +1,7 @@
 #pragma once
 
 #include "btf.h"
+#include "kfuncs.h"
 #include <memory>
 #include <optional>
 #include <string>
@@ -88,7 +89,6 @@ public:
   BPFfeature& operator=(BPFfeature&&) = delete;
 
   int instruction_limit();
-  bool has_loop();
   bool has_btf();
   bool has_btf_func_global();
   bool has_map_batch();
@@ -96,19 +96,20 @@ public:
   bool has_uprobe_refcnt();
   bool has_kprobe_multi();
   bool has_uprobe_multi();
-  bool has_kfunc();
+  bool has_fentry();
   bool has_skb_output();
-  bool has_raw_tp_special();
-  bool has_prog_kfunc();
+  bool has_prog_fentry();
   bool has_module_btf();
   bool has_iter(std::string name);
+  bool has_kernel_dwarf();
+
+  bool has_kernel_func(Kfunc kfunc);
 
   std::string report(void);
 
   DEFINE_MAP_TEST(array, libbpf::BPF_MAP_TYPE_ARRAY);
   DEFINE_MAP_TEST(hash, libbpf::BPF_MAP_TYPE_HASH);
   DEFINE_MAP_TEST(percpu_array, libbpf::BPF_MAP_TYPE_PERCPU_ARRAY);
-  DEFINE_MAP_TEST(percpu_hash, libbpf::BPF_MAP_TYPE_PERCPU_HASH);
   DEFINE_MAP_TEST(stack_trace, libbpf::BPF_MAP_TYPE_STACK_TRACE);
   DEFINE_MAP_TEST(perf_event_array, libbpf::BPF_MAP_TYPE_PERF_EVENT_ARRAY);
   DEFINE_MAP_TEST(ringbuf, libbpf::BPF_MAP_TYPE_RINGBUF);
@@ -126,12 +127,13 @@ public:
   DEFINE_HELPER_TEST(get_func_ip, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(jiffies64, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(for_each_map_elem, libbpf::BPF_PROG_TYPE_KPROBE);
+  DEFINE_HELPER_TEST(get_ns_current_pid_tgid, libbpf::BPF_PROG_TYPE_KPROBE);
+  DEFINE_HELPER_TEST(map_lookup_percpu_elem, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_PROG_TEST(kprobe, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_PROG_TEST(tracepoint, libbpf::BPF_PROG_TYPE_TRACEPOINT);
   DEFINE_PROG_TEST(perf_event, libbpf::BPF_PROG_TYPE_PERF_EVENT);
 
 protected:
-  std::optional<bool> has_loop_;
   std::optional<bool> has_d_path_;
   std::optional<int> insns_limit_;
   std::optional<bool> has_map_batch_;
@@ -139,10 +141,12 @@ protected:
   std::optional<bool> has_kprobe_multi_;
   std::optional<bool> has_uprobe_multi_;
   std::optional<bool> has_skb_output_;
-  std::optional<bool> has_raw_tp_special_;
-  std::optional<bool> has_prog_kfunc_;
+  std::optional<bool> has_prog_fentry_;
   std::optional<bool> has_module_btf_;
   std::optional<bool> has_btf_func_global_;
+  std::optional<bool> has_kernel_dwarf_;
+
+  std::unordered_map<Kfunc, bool> available_kernel_funcs_;
 
 private:
   bool detect_map(libbpf::bpf_map_type map_type);
