@@ -799,11 +799,10 @@ bool attach_reverse(const Probe &p)
     case ProbeType::uprobe:
     case ProbeType::uretprobe:
     case ProbeType::usdt:
-    case ProbeType::software:
     case ProbeType::fentry:
+    case ProbeType::fexit:
     case ProbeType::iter:
       return true;
-    case ProbeType::fexit:
     case ProbeType::kretprobe:
     case ProbeType::tracepoint:
     case ProbeType::profile:
@@ -812,6 +811,7 @@ bool attach_reverse(const Probe &p)
     case ProbeType::asyncwatchpoint:
     case ProbeType::hardware:
     case ProbeType::rawtracepoint:
+    case ProbeType::software:
       return false;
     case ProbeType::invalid:
       LOG(BUG) << "Unknown probe type";
@@ -1968,14 +1968,17 @@ std::unordered_set<std::string> BPFtrace::get_func_modules(
 #endif
 }
 
-const struct stat &BPFtrace::get_pidns_self_stat() const
+const std::optional<struct stat> &BPFtrace::get_pidns_self_stat() const
 {
-  static struct stat pidns = []() -> auto {
+  static std::optional<struct stat> pidns = []() -> std::optional<struct stat> {
     struct stat s;
-    if (::stat("/proc/self/ns/pid", &s))
+    if (::stat("/proc/self/ns/pid", &s)) {
+      if (errno == ENOENT)
+        return std::nullopt;
       throw std::runtime_error(
           std::string("Failed to stat /proc/self/ns/pid: ") +
           std::strerror(errno));
+    }
     return s;
   }();
 
