@@ -5,8 +5,6 @@
 #include "bpftrace.h"
 #include <clang-c/Index.h>
 
-#define CLANG_WORKAROUNDS_H "clang_workarounds.h"
-
 namespace bpftrace {
 
 namespace ast {
@@ -21,34 +19,26 @@ public:
 
 private:
   bool visit_children(CXCursor &cursor, BPFtrace &bpftrace);
-  /*
-   * The user might have written some struct definitions that rely on types
-   * supplied by BTF data.
-   *
-   * This method will pull out any forward-declared / incomplete struct
-   * definitions and return the types (in string form) of the unresolved types.
-   *
-   * Note that this method does not report "errors". This is because the user
-   * could have typo'd and actually referenced a non-existent type. Put
-   * differently, this method is best effort.
-   */
+  // The user might have written some struct definitions that rely on types
+  // supplied by BTF data.
+  //
+  // This method will pull out any forward-declared / incomplete struct
+  // definitions and return the types (in string form) of the unresolved types.
+  //
+  // Note that this method does not report "errors". This is because the user
+  // could have typo'd and actually referenced a non-existent type. Put
+  // differently, this method is best effort.
   std::unordered_set<std::string> get_incomplete_types();
-  /*
-   * Iteratively check for incomplete types, pull their definitions from BTF,
-   * and update the input files with the definitions.
-   */
+  // Iteratively check for incomplete types, pull their definitions from BTF,
+  // and update the input files with the definitions.
   void resolve_incomplete_types_from_btf(BPFtrace &bpftrace,
-                                         const ast::ProbeList *probes);
+                                         const ast::ProbeList &probes);
 
-  /*
-   * Collect names of types defined by typedefs that are in non-included
-   * headers as they may pose problems for clang parser.
-   */
+  // Collect names of types defined by typedefs that are in non-included
+  // headers as they may pose problems for clang parser.
   std::unordered_set<std::string> get_unknown_typedefs();
-  /*
-   * Iteratively check for unknown typedefs, pull their definitions from BTF,
-   * and update the input files with the definitions.
-   */
+  // Iteratively check for unknown typedefs, pull their definitions from BTF,
+  // and update the input files with the definitions.
   void resolve_unknown_typedefs_from_btf(BPFtrace &bpftrace);
 
   static std::optional<std::string> get_unknown_type(
@@ -85,11 +75,9 @@ private:
                                        unsigned num_unsaved_files,
                                        unsigned options);
 
-    /*
-     * Check diagnostics and collect all error messages.
-     * Return true if an error occurred. If bail_on_error is false, only fail
-     * on fatal errors.
-     */
+    // Check diagnostics and collect all error messages.
+    // Return true if an error occurred. If bail_on_error is false, only fail
+    // on fatal errors.
     bool check_diagnostics(bool bail_on_error);
 
     CXCursor get_translation_unit_cursor();
