@@ -8,22 +8,38 @@
 
 namespace bpftrace {
 
-Config::Config(bool has_cmd, bool bt_verbose) : bt_verbose_(bt_verbose)
+Config::Config(bool has_cmd)
 {
   config_map_ = {
     { ConfigKeyBool::cpp_demangle, { .value = true } },
     { ConfigKeyBool::lazy_symbolication, { .value = false } },
     { ConfigKeyBool::probe_inline, { .value = false } },
-    { ConfigKeyInt::log_size, { .value = (uint64_t)1000000 } },
-    { ConfigKeyInt::max_bpf_progs, { .value = (uint64_t)512 } },
-    { ConfigKeyInt::max_cat_bytes, { .value = (uint64_t)10240 } },
-    { ConfigKeyInt::max_map_keys, { .value = (uint64_t)4096 } },
-    { ConfigKeyInt::max_probes, { .value = (uint64_t)512 } },
-    { ConfigKeyInt::max_strlen, { .value = (uint64_t)64 } },
-    { ConfigKeyInt::max_type_res_iterations, { .value = (uint64_t)0 } },
-    { ConfigKeyInt::perf_rb_pages, { .value = (uint64_t)64 } },
+    { ConfigKeyBool::print_maps_on_exit, { .value = true } },
+#ifndef HAVE_BLAZESYM
+    { ConfigKeyBool::use_blazesym, { .value = false } },
+#else
+    { ConfigKeyBool::use_blazesym, { .value = true } },
+#endif
+    { ConfigKeyInt::log_size, { .value = static_cast<uint64_t>(1000000) } },
+    { ConfigKeyInt::max_bpf_progs, { .value = static_cast<uint64_t>(1024) } },
+    { ConfigKeyInt::max_cat_bytes, { .value = static_cast<uint64_t>(10240) } },
+    { ConfigKeyInt::max_map_keys, { .value = static_cast<uint64_t>(4096) } },
+    { ConfigKeyInt::max_probes, { .value = static_cast<uint64_t>(1024) } },
+    { ConfigKeyInt::max_strlen, { .value = static_cast<uint64_t>(1024) } },
+    { ConfigKeyInt::max_type_res_iterations,
+      { .value = static_cast<uint64_t>(0) } },
+    { ConfigKeyInt::on_stack_limit, { .value = static_cast<uint64_t>(32) } },
+    { ConfigKeyInt::perf_rb_pages, { .value = static_cast<uint64_t>(64) } },
     { ConfigKeyStackMode::default_, { .value = StackMode::bpftrace } },
     { ConfigKeyString::str_trunc_trailer, { .value = std::string("..") } },
+    { ConfigKeySymbolSource::default_,
+      { .value =
+#ifdef HAVE_LIBLLDB
+            ConfigSymbolSource::dwarf
+#else
+            ConfigSymbolSource::symbol_table
+#endif
+      } },
     { ConfigKeyMissingProbes::default_,
       { .value = ConfigMissingProbes::warn } },
     // by default, cache user symbols per program if ASLR is disabled on system
@@ -145,6 +161,21 @@ bool ConfigSetter::set_user_symbol_cache_type(const std::string &s)
   return config_.set(ConfigKeyUserSymbolCacheType::default_, usct, source_);
 }
 
+bool ConfigSetter::set_symbol_source_config(const std::string &s)
+{
+  ConfigSymbolSource source;
+  if (s == "dwarf") {
+    source = ConfigSymbolSource::dwarf;
+  } else if (s == "symbol_table") {
+    source = ConfigSymbolSource::symbol_table;
+  } else {
+    LOG(ERROR) << "Invalid value for symbol_source: valid values are "
+                  "\"dwarf\" and \"symbol_table\".";
+    return false;
+  }
+  return config_.set(ConfigKeySymbolSource::default_, source, source_);
+}
+
 bool ConfigSetter::set_missing_probes_config(const std::string &s)
 {
   ConfigMissingProbes mp;
