@@ -26,6 +26,8 @@ enum class ConfigKeyBool {
   cpp_demangle,
   lazy_symbolication,
   probe_inline,
+  print_maps_on_exit,
+  use_blazesym,
 };
 
 enum class ConfigKeyInt {
@@ -36,6 +38,7 @@ enum class ConfigKeyInt {
   max_probes,
   max_strlen,
   max_type_res_iterations,
+  on_stack_limit,
   perf_rb_pages,
 };
 
@@ -51,6 +54,15 @@ enum class ConfigKeyUserSymbolCacheType {
   default_,
 };
 
+enum class ConfigSymbolSource {
+  symbol_table,
+  dwarf,
+};
+
+enum class ConfigKeySymbolSource {
+  default_,
+};
+
 enum class ConfigMissingProbes {
   ignore,
   warn,
@@ -66,6 +78,7 @@ typedef std::variant<ConfigKeyBool,
                      ConfigKeyString,
                      ConfigKeyStackMode,
                      ConfigKeyUserSymbolCacheType,
+                     ConfigKeySymbolSource,
                      ConfigKeyMissingProbes>
     ConfigKey;
 
@@ -82,11 +95,15 @@ const std::map<std::string, ConfigKey> CONFIG_KEY_MAP = {
   { "max_probes", ConfigKeyInt::max_probes },
   { "max_strlen", ConfigKeyInt::max_strlen },
   { "max_type_res_iterations", ConfigKeyInt::max_type_res_iterations },
+  { "on_stack_limit", ConfigKeyInt::on_stack_limit },
   { "perf_rb_pages", ConfigKeyInt::perf_rb_pages },
   { "probe_inline", ConfigKeyBool::probe_inline },
   { "stack_mode", ConfigKeyStackMode::default_ },
   { "str_trunc_trailer", ConfigKeyString::str_trunc_trailer },
+  { "symbol_source", ConfigKeySymbolSource::default_ },
   { "missing_probes", ConfigKeyMissingProbes::default_ },
+  { "print_maps_on_exit", ConfigKeyBool::print_maps_on_exit },
+  { "use_blazesym", ConfigKeyBool::use_blazesym },
 };
 
 // These are not tracked by the config class
@@ -102,13 +119,14 @@ struct ConfigValue {
                std::string,
                StackMode,
                UserSymbolCacheType,
+               ConfigSymbolSource,
                ConfigMissingProbes>
       value;
 };
 
 class Config {
 public:
-  explicit Config(bool has_cmd = false, bool bt_verbose = false);
+  explicit Config(bool has_cmd = false);
 
   bool get(ConfigKeyBool key) const
   {
@@ -135,6 +153,11 @@ public:
     return get<UserSymbolCacheType>(key);
   }
 
+  ConfigSymbolSource get(ConfigKeySymbolSource key) const
+  {
+    return get<ConfigSymbolSource>(key);
+  }
+
   ConfigMissingProbes get(ConfigKeyMissingProbes key) const
   {
     return get<ConfigMissingProbes>(key);
@@ -181,7 +204,6 @@ private:
 private:
   bool can_set(ConfigSource prevSource, ConfigSource);
   bool is_aslr_enabled();
-  bool bt_verbose_ = false;
 
   std::map<ConfigKey, ConfigValue> config_map_;
 };
@@ -189,7 +211,7 @@ private:
 class ConfigSetter {
 public:
   explicit ConfigSetter(Config &config, ConfigSource source)
-      : config_(config), source_(source){};
+      : config_(config), source_(source) {};
 
   bool set(ConfigKeyBool key, bool val)
   {
@@ -223,6 +245,7 @@ public:
 
   bool set_stack_mode(const std::string &s);
   bool set_user_symbol_cache_type(const std::string &s);
+  bool set_symbol_source_config(const std::string &s);
   bool set_missing_probes_config(const std::string &s);
 
   Config &config_;
