@@ -5,9 +5,10 @@ namespace bpftrace {
 std::string logtype_str(LogType t)
 {
   switch (t) {
-    // clang-format off
+      // clang-format off
     case LogType::DEBUG   : return "";
     case LogType::V1      : return "";
+    case LogType::HINT    : return "HINT: ";
     case LogType::WARNING : return "WARNING: ";
     case LogType::ERROR   : return "ERROR: ";
     case LogType::BUG     : return "BUG: ";
@@ -19,10 +20,11 @@ std::string logtype_str(LogType t)
 
 Log::Log()
 {
-  enabled_map_[LogType::ERROR] = true;
-  enabled_map_[LogType::WARNING] = true;
-  enabled_map_[LogType::V1] = false;
   enabled_map_[LogType::DEBUG] = true;
+  enabled_map_[LogType::V1] = false;
+  enabled_map_[LogType::HINT] = true;
+  enabled_map_[LogType::WARNING] = true;
+  enabled_map_[LogType::ERROR] = true;
   enabled_map_[LogType::BUG] = true;
 }
 
@@ -37,28 +39,45 @@ void Log::take_input(LogType type,
                      std::ostream& out,
                      std::string&& input)
 {
-  auto print_out = [&]() { out << logtype_str(type) << input << std::endl; };
-
   if (loc) {
     if (src_.empty()) {
       std::cerr << "Log: cannot resolve location before calling set_source()."
                 << std::endl;
-      print_out();
+      out << log_format_output(type, std::move(input));
     } else if (loc->begin.line == 0) {
       std::cerr << "Log: invalid location." << std::endl;
-      print_out();
+      out << log_format_output(type, std::move(input));
     } else if (loc->begin.line > loc->end.line) {
       std::cerr << "Log: loc.begin > loc.end: " << loc->begin << ":" << loc->end
                 << std::endl;
-      print_out();
+      out << log_format_output(type, std::move(input));
     } else {
       log_with_location(type, loc.value(), out, input);
     }
   } else {
-    print_out();
+    out << log_format_output(type, std::move(input));
   }
 }
 
+std::string Log::log_format_output(LogType type, std::string&& input)
+{
+  if (!is_colorize_) {
+    return logtype_str(type) + std::move(input) + '\n';
+  }
+  std::string color;
+  switch (type) {
+    case LogType::ERROR:
+      color = LogColor::RED;
+      break;
+    case LogType::WARNING:
+      color = LogColor::YELLOW;
+      break;
+    default:
+      return logtype_str(type) + std::move(input) + '\n';
+  }
+  return color + logtype_str(type) + std::move(input) + LogColor::RESET + '\n';
+}
+
 const std::string Log::get_source_line(unsigned int n)
 {
   // Get the Nth source line (N is 0-based). Return an empty string if it
@@ -80,6 +99,23 @@ void Log::log_with_location(LogType type,
                             std::ostream& out,
                             const std::string& m)
 {
+  const char* color_begin = LogColor::DEFAULT;
+  const char* color_end = LogColor::DEFAULT;
+  if (is_colorize_) {
+    switch (type) {
+      case LogType::ERROR:
+        color_begin = LogColor::RED;
+        color_end = LogColor::RESET;
+        break;
+      case LogType::WARNING:
+        color_begin = LogColor::YELLOW;
+        color_end = LogColor::RESET;
+        break;
+      default:
+        break;
+    }
+  }
+  out << color_begin;
   if (filename_.size()) {
     out << filename_ << ":";
   }
@@ -91,36 +127,35 @@ void Log::log_with_location(LogType type,
     msg.pop_back();
   }
 
-  /* For a multi line error only the line range is printed:
-     <filename>:<start_line>-<end_line>: ERROR: <message>
-  */
+  // For a multi line error only the line range is printed:
+  //     <filename>:<start_line>-<end_line>: ERROR: <message>
   if (l.begin.line < l.end.line) {
     out << l.begin.line << "-" << l.end.line << ": " << typestr << msg
-        << std::endl;
+        << std::endl
+        << color_end;
     return;
   }
 
-  /*
-    For a single line error the format is:
-
-    <filename>:<line>:<start_col>-<end_col>: ERROR: <message>
-    <source line>
-    <marker>
-
-    E.g.
-
-    file.bt:1:10-20: error: <message>
-    i:s:1   /1 < "str"/
-            ~~~~~~~~~~
-  */
+  // For a single line error the format is:
+  //
+  // <filename>:<line>:<start_col>-<end_col>: ERROR: <message>
+  // <source line>
+  // <marker>
+  //
+  // E.g.
+  //
+  // file.bt:1:10-20: error: <message>
+  // i:s:1   /1 < "str"/
+  //         ~~~~~~~~~~
   out << l.begin.line << ":" << l.begin.column << "-" << l.end.column;
-  out << ": " << typestr << msg << std::endl;
+  out << ": " << typestr << msg << std::endl << color_end;
 
   // for bpftrace::position, valid line# starts from 1
   std::string srcline = get_source_line(l.begin.line - 1);
 
-  if (srcline == "")
+  if (srcline == "") {
     return;
+  }
 
   // To get consistent printing all tabs will be replaced with 4 spaces
   for (auto c : srcline) {
@@ -161,7 +196,7 @@ LogStream::LogStream(const std::string& file,
 LogStream::LogStream(const std::string& file,
                      int line,
                      LogType type,
-                     const location& loc,
+                     std::optional<location> loc,
                      std::ostream& out)
     : sink_(Log::get()),
       type_(type),
