@@ -12,6 +12,7 @@
 #include "dwarf_parser.h"
 #include "log.h"
 #include "probe_matcher.h"
+#include "scopeguard.h"
 #include "tracefs.h"
 #include "utils.h"
 
@@ -31,9 +32,7 @@ static int add_symbol(const char* symname,
   return 0;
 }
 
-/*
- * Finds all matches of search_input in the provided input stream.
- */
+// Finds all matches of search_input in the provided input stream.
 std::set<std::string> ProbeMatcher::get_matches_in_stream(
     const std::string& search_input,
     std::istream& symbol_stream,
@@ -43,6 +42,17 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
   bool start_wildcard, end_wildcard;
   auto tokens = get_wildcard_tokens(search_input, start_wildcard, end_wildcard);
 
+  // Since demangled_name contains function parameters, we need to remove
+  // them unless the user specified '(' in the search input (i.e. wants
+  // to match against the parameters explicitly).
+  // Only used for C++ when demangling is enabled.
+  auto has_parameter = [](const std::string& token) {
+    return token.find('(') != std::string::npos;
+  };
+  const bool truncate_parameters = std::none_of(tokens.begin(),
+                                                tokens.end(),
+                                                has_parameter);
+
   std::string line;
   std::set<std::string> matches;
   while (std::getline(symbol_stream, line, delim)) {
@@ -56,25 +66,19 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
           char* demangled_name = cxxdemangle(fun_line.c_str());
           if (!demangled_name)
             continue;
+          SCOPE_EXIT
+          {
+            ::free(demangled_name);
+          };
 
           // Match against the demanled name.
-          // Since demangled_name contains function arguments, we need to remove
-          // them unless the user specified '(' in the search input (i.e. wants
-          // to match against the arguments explicitly).
           std::string match_line = prefix + demangled_name;
-          if (std::all_of(tokens.begin(),
-                          tokens.end(),
-                          [&](const std::string& token) {
-                            return token.find("(") == std::string::npos;
-                          })) {
-            match_line = match_line.substr(0, match_line.find_last_of("("));
+          if (truncate_parameters) {
+            erase_parameter_list(match_line);
           }
 
-          if (!wildcard_match(
+          if (wildcard_match(
                   match_line, tokens, start_wildcard, end_wildcard)) {
-            free(demangled_name);
-          } else {
-            free(demangled_name);
             goto out;
           }
         }
@@ -91,11 +95,9 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
   return matches;
 }
 
-/*
- * Get matches of search_input (containing a wildcard) for a given probe_type.
- * probe_type determines where to take the candidate matches from.
- * Some probe types (e.g. uprobe) require target to be specified.
- */
+// Get matches of search_input (containing a wildcard) for a given probe_type.
+// probe_type determines where to take the candidate matches from.
+// Some probe types (e.g. uprobe) require target to be specified.
 std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     const ProbeType& probe_type,
     const std::string& target,
@@ -141,8 +143,8 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
       symbol_stream = get_symbols_from_list(HW_PROBE_LIST);
       break;
     }
-    case ProbeType::kfunc:
-    case ProbeType::kretfunc: {
+    case ProbeType::fentry:
+    case ProbeType::fexit: {
       // If BTF is not parsed, yet, read available_filter_functions instead.
       // This is useful as we will use the result to extract the list of
       // potentially used kernel modules and then only parse BTF for them.
@@ -179,6 +181,8 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
       symbol_stream = std::make_unique<std::istringstream>(ret);
       break;
     }
+    case ProbeType::special:
+      return { target + ":" };
     default:
       return {};
   }
@@ -191,9 +195,7 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     return {};
 }
 
-/*
- * Find all matches of search_input in set
- */
+// Find all matches of search_input in set
 std::set<std::string> ProbeMatcher::get_matches_in_set(
     const std::string& search_input,
     const std::set<std::string>& set)
@@ -325,10 +327,8 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_list(
   return std::make_unique<std::istringstream>(symbols);
 }
 
-/*
- * Get list of kernel probe types for the purpose of listing.
- * Ignore return probes and aliases.
- */
+// Get list of kernel probe types for the purpose of listing.
+// Ignore return probes and aliases.
 std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
 {
   std::string probes;
@@ -336,9 +336,9 @@ std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
     if (!p.show_in_kernel_list) {
       continue;
     }
-    if (p.type == ProbeType::kfunc) {
-      // kfunc must be available
-      if (bpftrace_->feature_->has_kfunc())
+    if (p.type == ProbeType::fentry) {
+      // fentry must be available
+      if (bpftrace_->feature_->has_fentry())
         probes += p.name + "\n";
     } else {
       probes += p.name + "\n";
@@ -348,10 +348,8 @@ std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
   return std::make_unique<std::istringstream>(probes);
 }
 
-/*
- * Get list of userspace probe types for the purpose of listing.
- * Ignore return probes.
- */
+// Get list of userspace probe types for the purpose of listing.
+// Ignore return probes.
 std::unique_ptr<std::istream> ProbeMatcher::userspace_probe_list()
 {
   std::string probes;
@@ -445,16 +443,16 @@ FuncParamLists ProbeMatcher::get_uprobe_params(
 
 void ProbeMatcher::list_probes(ast::Program* prog)
 {
-  for (auto* probe : *prog->probes) {
-    for (auto* ap : *probe->attach_points) {
+  for (auto* probe : prog->probes) {
+    for (auto* ap : probe->attach_points) {
       auto matches = get_matches_for_ap(*ap);
       auto probe_type = probetype(ap->provider);
       FuncParamLists param_lists;
       if (bt_verbose) {
         if (probe_type == ProbeType::tracepoint)
           param_lists = get_tracepoints_params(matches);
-        else if (probe_type == ProbeType::kfunc ||
-                 probe_type == ProbeType::kretfunc)
+        else if (probe_type == ProbeType::fentry ||
+                 probe_type == ProbeType::fexit)
           param_lists = bpftrace_->btf_->get_params(matches);
         else if (probe_type == ProbeType::iter)
           param_lists = get_iters_params(matches);
@@ -467,6 +465,10 @@ void ProbeMatcher::list_probes(ast::Program* prog)
         if (ap->lang == "cpp") {
           std::string target = erase_prefix(match_print);
           char* demangled_name = cxxdemangle(match_print.c_str());
+          SCOPE_EXIT
+          {
+            ::free(demangled_name);
+          };
 
           // demangled name may contain symbols not accepted by the attach point
           // parser, so surround it with quotes to make the entry directly
@@ -480,7 +482,7 @@ void ProbeMatcher::list_probes(ast::Program* prog)
         std::cout << probe_type << ":" << match_print << std::endl;
         if (bt_verbose) {
           for (auto& param : param_lists[match])
-            LOG(V1) << "    " << param;
+            std::cout << "    " << param << std::endl;
         }
       }
     }
@@ -511,8 +513,8 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
     case ProbeType::watchpoint:
     case ProbeType::asyncwatchpoint:
     case ProbeType::tracepoint:
-    case ProbeType::kfunc:
-    case ProbeType::kretfunc: {
+    case ProbeType::fentry:
+    case ProbeType::fexit: {
       // Do not expand "target:" as that would match all functions in target.
       // This may occur when an absolute address is given instead of a function.
       if (attach_point.func.empty())
