@@ -12,7 +12,6 @@
 
 #include "format_string.h"
 #include "location.hh"
-#include "mapkey.h"
 #include "struct.h"
 #include "types.h"
 
@@ -49,7 +48,7 @@ private:
 };
 
 struct MapInfo {
-  MapKey key;
+  SizedType key_type;
   SizedType value_type;
   std::optional<LinearHistogramArgs> lhist_args;
   std::optional<int> hist_bits_arg;
@@ -60,7 +59,7 @@ private:
   template <typename Archive>
   void serialize(Archive &archive)
   {
-    archive(key, value_type, lhist_args, hist_bits_arg, id);
+    archive(key_type, value_type, lhist_args, hist_bits_arg, id);
   }
 };
 
@@ -86,45 +85,62 @@ public:
   void load_state(const uint8_t *ptr, size_t len);
 
   // Async argument metadata
+  std::vector<std::tuple<FormatString, std::vector<Field>>> printf_args;
   std::vector<std::tuple<FormatString, std::vector<Field>>> system_args;
-  // mapped_printf_args stores seq_printf, debugf arguments
-  std::vector<std::tuple<FormatString, std::vector<Field>>> mapped_printf_args;
-  // mapped_printf_ids stores the starting indices and length of each format
-  // string in the data map of MapType::MappedPrintfData
-  std::vector<std::tuple<int, int>> mapped_printf_ids;
+  // fmt strings for BPF helpers (bpf_seq_printf, bpf_trace_printk)
+  std::vector<FormatString> bpf_print_fmts;
+  std::vector<std::tuple<FormatString, std::vector<Field>>> cat_args;
   std::vector<std::string> join_args;
   std::vector<std::string> time_args;
   std::vector<std::string> strftime_args;
   std::vector<std::string> cgroup_path_args;
-  std::vector<std::tuple<FormatString, std::vector<Field>>> cat_args;
   std::vector<SizedType> non_map_print_args;
   std::vector<std::tuple<std::string, long>> skboutput_args_;
+  // While max fmtstring args size is not used at runtime, the size
+  // calculation requires taking into account struct alignment semantics,
+  // and that is tricky enough that we want to minimize repetition of
+  // such logic in the codebase. So keep it in resource analysis
+  // rather than duplicating it in CodegenResources.
+  uint64_t max_fmtstring_args_size = 0;
+
+  // Required for sizing of tuple scratch buffer
+  size_t tuple_buffers = 0;
+  size_t max_tuple_size = 0;
+
+  // Required for sizing of string scratch buffer
+  size_t str_buffers = 0;
+
+  // Required for sizing of map value scratch buffers
+  size_t read_map_value_buffers = 0;
+  size_t max_read_map_value_size = 0;
+  size_t max_write_map_value_size = 0;
+
+  // Required for sizing of variable scratch buffers
+  size_t variable_buffers = 0;
+  size_t max_variable_size = 0;
+
+  // Required for sizing of map key scratch buffers
+  size_t map_key_buffers = 0;
+  size_t max_map_key_size = 0;
 
   // Async argument metadata that codegen creates. Ideally ResourceAnalyser
   // pass should be collecting this, but it's complex to move the logic.
   //
   // Don't add more async arguments here!.
   std::unordered_map<int64_t, struct HelperErrorInfo> helper_error_info;
-  // `printf_args` is created here but the field offsets are fixed up
-  // by codegen -- only codegen knows data layout to compute offsets
-  std::vector<std::tuple<FormatString, std::vector<Field>>> printf_args;
   std::vector<std::string> probe_ids;
 
   // Map metadata
   std::map<std::string, MapInfo> maps_info;
-  std::unordered_set<StackType> stackid_maps;
-  bool needs_join_map = false;
-  bool needs_elapsed_map = false;
-  bool needs_data_map = false;
+  std::unordered_set<bpftrace::globalvars::GlobalVar> needed_global_vars;
   bool needs_perf_event_map = false;
-  uint32_t str_buffers = 0;
 
   // Probe metadata
   //
   // Probe metadata that codegen creates. Ideally ResourceAnalyser pass should
   // be collecting this, but it's complex to move the logic.
   std::vector<Probe> probes;
-  std::vector<Probe> special_probes;
+  std::unordered_map<std::string, Probe> special_probes;
   std::vector<Probe> watchpoint_probes;
 
   // List of probes using userspace symbol resolution
@@ -136,8 +152,7 @@ private:
   void serialize(Archive &archive)
   {
     archive(system_args,
-            mapped_printf_args,
-            mapped_printf_ids,
+            bpf_print_fmts,
             join_args,
             time_args,
             strftime_args,
@@ -148,10 +163,7 @@ private:
             printf_args,
             probe_ids,
             maps_info,
-            stackid_maps,
-            needs_join_map,
-            needs_elapsed_map,
-            needs_data_map,
+            needed_global_vars,
             needs_perf_event_map,
             probes,
             special_probes);
