@@ -24,23 +24,23 @@ enum class Type : uint8_t {
   // clang-format off
   none,
   voidtype,
-  integer,
+  integer, // int is a protected keyword
   pointer,
+  reference,
   record, // struct/union, as struct is a protected keyword
-  hist,
-  lhist,
-  count,
-  sum,
-  min,
-  max,
-  avg,
-  stats,
-  kstack,
-  ustack,
+  hist_t,
+  lhist_t,
+  count_t,
+  sum_t,
+  min_t,
+  max_t,
+  avg_t,
+  stats_t,
+  kstack_t,
+  ustack_t,
   string,
-  ksym,
-  usym,
-  probe,
+  ksym_t,
+  usym_t,
   username,
   inet,
   stack_mode,
@@ -49,8 +49,8 @@ enum class Type : uint8_t {
   tuple,
   timestamp,
   mac_address,
-  cgroup_path,
-  strerror,
+  cgroup_path_t,
+  strerror_t,
   timestamp_mode,
   // clang-format on
 };
@@ -59,10 +59,12 @@ enum class AddrSpace : uint8_t {
   none,
   kernel,
   user,
+  bpf,
 };
 
 std::ostream &operator<<(std::ostream &os, Type type);
 std::ostream &operator<<(std::ostream &os, AddrSpace as);
+std::string to_string(Type ty);
 
 enum class UserSymbolCacheType {
   per_pid,
@@ -148,7 +150,7 @@ private:
   size_t size_bits_ = 0;                    // size in bits
   std::shared_ptr<SizedType> element_type_; // for "container" and pointer
                                             // (like) types
-  std::string name_; // name of this type, for named types like struct
+  std::string name_; // name of this type, for named types like struct and enum
   std::weak_ptr<Struct> inner_struct_; // inner struct for records and tuples
                                        // the actual Struct object is owned by
                                        // StructManager
@@ -157,6 +159,7 @@ private:
   bool ctx_ = false;                                   // Is bpf program context
   std::unordered_set<std::string> btf_type_tags_ = {}; // Only populated for
                                                        // Type::pointer
+  size_t num_elements_ = 0; // Only populated for array types
 
   friend class cereal::access;
   template <typename Archive>
@@ -179,9 +182,7 @@ private:
   }
 
 public:
-  /**
-     Tuple/struct accessors
-  */
+  // Tuple/struct accessors
   std::vector<Field> &GetFields() const;
   bool HasField(const std::string &name) const;
   const Field &GetField(const std::string &name) const;
@@ -189,14 +190,10 @@ public:
   ssize_t GetFieldCount() const;
   std::weak_ptr<const Struct> GetStruct() const;
 
-  /**
-     Required alignment for this type when used inside a tuple
-   */
+  // Required alignment for this type when used inside a tuple
   ssize_t GetInTupleAlignment() const;
 
-  /**
-     Dump the underlying structure for debug purposes
-  */
+  // Dump the underlying structure for debug purposes
   void DumpStructure(std::ostream &os);
 
   AddrSpace GetAS() const
@@ -239,6 +236,9 @@ public:
   bool operator==(const SizedType &t) const;
   bool operator!=(const SizedType &t) const;
   bool IsSameType(const SizedType &t) const;
+  // This is primarily for Tuples which have equal total size (due to padding)
+  // but their individual elements have different sizes.
+  bool IsSameSizeRecursive(const SizedType &t) const;
   bool FitsInto(const SizedType &t) const;
 
   bool IsPrintableTy()
@@ -248,6 +248,11 @@ public:
            (!IsCtxAccess() || is_funcarg); // args builtin is printable
   }
 
+  void SetSign(bool is_signed)
+  {
+    is_signed_ = is_signed;
+  }
+
   bool IsSigned(void) const;
 
   size_t GetSize() const
@@ -286,13 +291,17 @@ public:
 
   size_t GetNumElements() const
   {
-    assert(IsArrayTy() || IsStringTy());
-    return IsStringTy() ? size_bits_ : size_bits_ / element_type_->size_bits_;
+    assert(IsArrayTy());
+    // For arrays we can't just do size_bits_ / element_type.GetSize()
+    // because we might not know the size of the element type (it may be 0)
+    // if it's being resolved in a later AST pass e.g. for an imported type:
+    // `let $x: struct Foo[10];`
+    return num_elements_;
   };
 
   const std::string GetName() const
   {
-    assert(IsRecordTy());
+    assert(IsRecordTy() || IsEnumTy());
     return name_;
   }
 
@@ -313,6 +322,12 @@ public:
     return element_type_.get();
   }
 
+  const SizedType *GetDereferencedTy() const
+  {
+    assert(IsRefTy());
+    return element_type_.get();
+  }
+
   bool IsBoolTy() const
   {
     return type_ == Type::integer && size_bits_ == 1;
@@ -321,10 +336,18 @@ public:
   {
     return type_ == Type::pointer;
   };
+  bool IsRefTy() const
+  {
+    return type_ == Type::reference;
+  };
   bool IsIntTy() const
   {
     return type_ == Type::integer;
   };
+  bool IsEnumTy() const
+  {
+    return IsIntTy() && name_.size();
+  }
   bool IsNoneTy(void) const
   {
     return type_ == Type::none;
@@ -339,43 +362,43 @@ public:
   };
   bool IsHistTy(void) const
   {
-    return type_ == Type::hist;
+    return type_ == Type::hist_t;
   };
   bool IsLhistTy(void) const
   {
-    return type_ == Type::lhist;
+    return type_ == Type::lhist_t;
   };
   bool IsCountTy(void) const
   {
-    return type_ == Type::count;
+    return type_ == Type::count_t;
   };
   bool IsSumTy(void) const
   {
-    return type_ == Type::sum;
+    return type_ == Type::sum_t;
   };
   bool IsMinTy(void) const
   {
-    return type_ == Type::min;
+    return type_ == Type::min_t;
   };
   bool IsMaxTy(void) const
   {
-    return type_ == Type::max;
+    return type_ == Type::max_t;
   };
   bool IsAvgTy(void) const
   {
-    return type_ == Type::avg;
+    return type_ == Type::avg_t;
   };
   bool IsStatsTy(void) const
   {
-    return type_ == Type::stats;
+    return type_ == Type::stats_t;
   };
   bool IsKstackTy(void) const
   {
-    return type_ == Type::kstack;
+    return type_ == Type::kstack_t;
   };
   bool IsUstackTy(void) const
   {
-    return type_ == Type::ustack;
+    return type_ == Type::ustack_t;
   };
   bool IsStringTy(void) const
   {
@@ -383,15 +406,11 @@ public:
   };
   bool IsKsymTy(void) const
   {
-    return type_ == Type::ksym;
+    return type_ == Type::ksym_t;
   };
   bool IsUsymTy(void) const
   {
-    return type_ == Type::usym;
-  };
-  bool IsProbeTy(void) const
-  {
-    return type_ == Type::probe;
+    return type_ == Type::usym_t;
   };
   bool IsUsernameTy(void) const
   {
@@ -431,11 +450,11 @@ public:
   };
   bool IsCgroupPathTy(void) const
   {
-    return type_ == Type::cgroup_path;
+    return type_ == Type::cgroup_path_t;
   };
   bool IsStrerrorTy(void) const
   {
-    return type_ == Type::strerror;
+    return type_ == Type::strerror_t;
   };
   bool IsTimestampModeTy(void) const
   {
@@ -443,24 +462,40 @@ public:
   }
   bool IsCastableMapTy() const
   {
-    return type_ == Type::count || type_ == Type::sum || type_ == Type::max ||
-           type_ == Type::min;
+    return type_ == Type::count_t || type_ == Type::sum_t ||
+           type_ == Type::max_t || type_ == Type::min_t || type_ == Type::avg_t;
   }
   bool IsMapIterableTy() const
   {
-    return !(type_ == Type::avg || type_ == Type::hist ||
-             type_ == Type::lhist || type_ == Type::stats);
+    return !IsMultiOutputMapTy();
   }
 
-  friend std::ostream &operator<<(std::ostream &, const SizedType &);
-  friend std::ostream &operator<<(std::ostream &, Type);
+  // These are special map value types that can't be reduced to a single value
+  // and output multiple lines when printed
+  bool IsMultiOutputMapTy() const
+  {
+    return type_ == Type::hist_t || type_ == Type::lhist_t ||
+           type_ == Type::stats_t;
+  }
+
+  bool NeedsPercpuMap() const;
+
+  friend std::string typestr(const SizedType &type);
+
+  void IntoPointer()
+  {
+    assert(IsRefTy());
+    type_ = Type::pointer;
+  }
 
   // Factories
 
+  friend SizedType CreateEnum(size_t bits, const std::string &name);
   friend SizedType CreateArray(size_t num_elements,
                                const SizedType &element_type);
 
   friend SizedType CreatePointer(const SizedType &pointee_type, AddrSpace as);
+  friend SizedType CreateReference(const SizedType &pointee_type, AddrSpace as);
   friend SizedType CreateRecord(const std::string &name,
                                 std::weak_ptr<Struct> record);
   friend SizedType CreateInteger(size_t bits, bool is_signed);
@@ -482,11 +517,14 @@ SizedType CreateUInt8();
 SizedType CreateUInt16();
 SizedType CreateUInt32();
 SizedType CreateUInt64();
+SizedType CreateEnum(size_t bits, const std::string &name);
 
 SizedType CreateString(size_t size);
 SizedType CreateArray(size_t num_elements, const SizedType &element_type);
 SizedType CreatePointer(const SizedType &pointee_type,
                         AddrSpace as = AddrSpace::none);
+SizedType CreateReference(const SizedType &referred_type,
+                          AddrSpace as = AddrSpace::none);
 
 SizedType CreateRecord(const std::string &name, std::weak_ptr<Struct> record);
 SizedType CreateTuple(std::weak_ptr<Struct> tuple);
@@ -500,7 +538,6 @@ SizedType CreateSum(bool is_signed);
 SizedType CreateCount(bool is_signed);
 SizedType CreateAvg(bool is_signed);
 SizedType CreateStats(bool is_signed);
-SizedType CreateProbe();
 SizedType CreateUsername();
 SizedType CreateInet(size_t size);
 SizedType CreateLhist();
@@ -531,8 +568,8 @@ enum class ProbeType {
   hardware,
   watchpoint,
   asyncwatchpoint,
-  kfunc,
-  kretfunc,
+  fentry,
+  fexit,
   iter,
   rawtracepoint,
 };
@@ -550,33 +587,64 @@ struct ProbeItem {
 };
 
 const std::vector<ProbeItem> PROBE_LIST = {
-  { "kprobe", { "k" }, ProbeType::kprobe, .show_in_kernel_list = true },
-  { "kretprobe", { "kr" }, ProbeType::kretprobe },
-  { "uprobe", { "u" }, ProbeType::uprobe, .show_in_userspace_list = true },
-  { "uretprobe", { "ur" }, ProbeType::uretprobe },
-  { "usdt", { "U" }, ProbeType::usdt, .show_in_userspace_list = true },
-  { "BEGIN", { "BEGIN" }, ProbeType::special },
-  { "END", { "END" }, ProbeType::special },
-  { "tracepoint", { "t" }, ProbeType::tracepoint, .show_in_kernel_list = true },
-  { "profile", { "p" }, ProbeType::profile },
-  { "interval", { "i" }, ProbeType::interval },
-  { "software", { "s" }, ProbeType::software, .show_in_kernel_list = true },
-  { "hardware", { "h" }, ProbeType::hardware, .show_in_kernel_list = true },
-  { "watchpoint", { "w" }, ProbeType::watchpoint },
-  { "asyncwatchpoint", { "aw" }, ProbeType::asyncwatchpoint },
-  { "kfunc", { "f", "fentry" }, ProbeType::kfunc, .show_in_kernel_list = true },
-  { "kretfunc", { "fr", "fexit" }, ProbeType::kretfunc },
-  { "iter", { "it" }, ProbeType::iter, .show_in_kernel_list = true },
-  { "rawtracepoint",
-    { "rt" },
-    ProbeType::rawtracepoint,
+  { .name = "kprobe",
+    .aliases = { "k" },
+    .type = ProbeType::kprobe,
+    .show_in_kernel_list = true },
+  { .name = "kretprobe", .aliases = { "kr" }, .type = ProbeType::kretprobe },
+  { .name = "uprobe",
+    .aliases = { "u" },
+    .type = ProbeType::uprobe,
+    .show_in_userspace_list = true },
+  { .name = "uretprobe", .aliases = { "ur" }, .type = ProbeType::uretprobe },
+  { .name = "usdt",
+    .aliases = { "U" },
+    .type = ProbeType::usdt,
+    .show_in_userspace_list = true },
+  { .name = "BEGIN", .aliases = { "BEGIN" }, .type = ProbeType::special },
+  { .name = "END", .aliases = { "END" }, .type = ProbeType::special },
+  { .name = "self", .aliases = { "self" }, .type = ProbeType::special },
+  { .name = "tracepoint",
+    .aliases = { "t" },
+    .type = ProbeType::tracepoint,
+    .show_in_kernel_list = true },
+  { .name = "profile", .aliases = { "p" }, .type = ProbeType::profile },
+  { .name = "interval", .aliases = { "i" }, .type = ProbeType::interval },
+  { .name = "software",
+    .aliases = { "s" },
+    .type = ProbeType::software,
+    .show_in_kernel_list = true },
+  { .name = "hardware",
+    .aliases = { "h" },
+    .type = ProbeType::hardware,
+    .show_in_kernel_list = true },
+  { .name = "watchpoint", .aliases = { "w" }, .type = ProbeType::watchpoint },
+  { .name = "asyncwatchpoint",
+    .aliases = { "aw" },
+    .type = ProbeType::asyncwatchpoint },
+  { .name = "fentry",
+    .aliases = { "f", "kfunc" },
+    .type = ProbeType::fentry,
+    .show_in_kernel_list = true },
+  { .name = "fexit",
+    .aliases = { "fr", "kretfunc" },
+    .type = ProbeType::fexit },
+  { .name = "iter",
+    .aliases = { "it" },
+    .type = ProbeType::iter,
+    .show_in_kernel_list = true },
+  { .name = "rawtracepoint",
+    .aliases = { "rt" },
+    .type = ProbeType::rawtracepoint,
     .show_in_kernel_list = true },
 };
 
 ProbeType probetype(const std::string &type);
 std::string addrspacestr(AddrSpace as);
 std::string typestr(Type t);
+std::string typestr(const SizedType &type);
 std::string expand_probe_name(const std::string &orig_name);
+std::string probetypeName(ProbeType t);
 
 struct Probe {
   ProbeType type;
@@ -652,6 +720,27 @@ uint64_t asyncactionint(AsyncAction a);
 
 enum class PositionalParameterType { positional, count };
 
+namespace globalvars {
+
+enum class GlobalVar {
+  // Number of online CPUs at runtime, used for metric aggregation for functions
+  // like sum and avg
+  NUM_CPUS,
+  // Max CPU ID returned by bpf_get_smp_processor_id, used for simulating
+  // per-CPU maps in read-write global variables
+  MAX_CPU_ID,
+  // Scratch buffers used to avoid BPF stack allocation limits
+  FMT_STRINGS_BUFFER,
+  TUPLE_BUFFER,
+  GET_STR_BUFFER,
+  READ_MAP_VALUE_BUFFER,
+  WRITE_MAP_VALUE_BUFFER,
+  VARIABLE_BUFFER,
+  MAP_KEY_BUFFER,
+};
+
+} // namespace globalvars
+
 } // namespace bpftrace
 
 // SizedType hash function
