@@ -1,42 +1,45 @@
 #include <algorithm>
 #include <array>
+#include <cerrno>
 #include <climits>
 #include <cmath>
 #include <cstring>
-#include <errno.h>
-#include <fcntl.h>
+#include <filesystem>
 #include <fstream>
-#include <gelf.h>
-#include <glob.h>
-#include <libelf.h>
 #include <limits>
-#include <link.h>
 #include <map>
 #include <memory>
+#include <optional>
+#include <ranges>
 #include <regex>
 #include <sstream>
 #include <string>
-#include <sys/auxv.h>
-#include <sys/stat.h>
 #include <system_error>
 #include <tuple>
-#include <unistd.h>
 #include <unordered_set>
 
-#include "bpftrace.h"
-#include "debugfs.h"
-#include "filesystem.h"
-#include "log.h"
-#include "probe_matcher.h"
-#include "tracefs.h"
-#include "utils.h"
 #include <bcc/bcc_elf.h>
 #include <bcc/bcc_syms.h>
 #include <bcc/bcc_usdt.h>
 #include <elf.h>
+#include <fcntl.h>
+#include <gelf.h>
+#include <glob.h>
+#include <libelf.h>
+#include <link.h>
+#include <linux/version.h>
+#include <sys/auxv.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <zlib.h>
 
-#include <linux/version.h>
+#include "bpftrace.h"
+#include "debugfs.h"
+#include "log.h"
+#include "probe_matcher.h"
+#include "scopeguard.h"
+#include "tracefs.h"
+#include "utils.h"
 
 namespace {
 
@@ -108,6 +111,72 @@ const struct vmlinux_location vmlinux_locs[] = {
   { nullptr, false },
 };
 
+std::optional<std::string> find_vmlinux(struct vmlinux_location const *locs,
+                                        struct symbol *sym)
+{
+  struct utsname uts;
+  if (uname(&uts) != 0)
+    return std::nullopt;
+
+  for (size_t i = 0; locs[i].path; ++i) {
+    auto &loc = locs[i];
+    if (loc.raw)
+      continue; // This file is for BTF. skip
+
+    // Format the loc.path with the current release.
+    char path[PATH_MAX] = {};
+    int bytes_written = std::snprintf(
+        path, sizeof(path), loc.path, uts.release);
+    if (bytes_written < 0) {
+      LOG(ERROR) << "Failed to format vmlinux path '" << loc.path << "' using "
+                 << uts.release;
+      continue;
+    } else if (static_cast<size_t>(bytes_written) > sizeof(path)) {
+      LOG(WARNING) << "Truncated format for vmlinux path '" << loc.path
+                   << "' using " << uts.release;
+      continue;
+    }
+
+    if (access(path, R_OK))
+      continue;
+
+    if (sym == nullptr) {
+      return path;
+    } else {
+      bcc_elf_symcb callback = !sym->name.empty() ? sym_name_cb
+                                                  : sym_address_cb;
+      struct bcc_symbol_option options = {
+        .use_debug_file = 0,
+        .check_debug_file_crc = 0,
+        .lazy_symbolize = 0,
+        .use_symbol_type = BCC_SYM_ALL_TYPES ^ (1 << STT_NOTYPE),
+      };
+      if (bcc_elf_foreach_sym(path, callback, &options, sym) == -1) {
+        LOG(ERROR) << "Failed to iterate over symbols in " << path;
+        continue;
+      }
+
+      if (sym->start) {
+        LOG(V1) << "vmlinux: using " << path;
+        return path;
+      }
+    }
+  }
+
+  return std::nullopt;
+}
+
+// find vmlinux file.
+// if sym is not null, check vmlinux contains the given symbol information
+std::optional<std::string> find_vmlinux(struct symbol *sym)
+{
+  struct vmlinux_location locs_env[] = {
+    { std::getenv("BPFTRACE_VMLINUX"), false },
+    { nullptr, false },
+  };
+  return find_vmlinux(locs_env[0].path ? locs_env : vmlinux_locs, sym);
+}
+
 static bool pid_in_different_mountns(int pid);
 static std::vector<std::string> resolve_binary_path(const std::string &cmd,
                                                     const char *env_paths,
@@ -250,7 +319,7 @@ void get_bool_env_var(const ::std::string &str,
   return;
 }
 
-std::optional<std_filesystem::path> find_in_path(std::string_view name)
+std::optional<std::filesystem::path> find_in_path(std::string_view name)
 {
   std::error_code ec;
 
@@ -260,25 +329,25 @@ std::optional<std_filesystem::path> find_in_path(std::string_view name)
 
   auto paths = split_string(path_env, ':', true);
   for (const auto &path : paths) {
-    auto fpath = std_filesystem::path(path) / name;
-    if (std_filesystem::exists(fpath, ec))
+    auto fpath = std::filesystem::path(path) / name;
+    if (std::filesystem::exists(fpath, ec))
       return fpath;
   }
 
   return std::nullopt;
 }
 
-std::optional<std_filesystem::path> find_near_self(std::string_view filename)
+std::optional<std::filesystem::path> find_near_self(std::string_view filename)
 {
   std::error_code ec;
-  auto exe = std_filesystem::read_symlink("/proc/self/exe", ec);
+  auto exe = std::filesystem::read_symlink("/proc/self/exe", ec);
   if (ec) {
     LOG(WARNING) << "Failed to resolve /proc/self/exe: " << ec;
     return std::nullopt;
   }
 
   exe.replace_filename(filename);
-  bool exists = std_filesystem::exists(exe, ec);
+  bool exists = std::filesystem::exists(exe, ec);
   if (!exists) {
     if (ec)
       LOG(WARNING) << "Failed to resolve stat " << exe << ": " << ec;
@@ -291,13 +360,13 @@ std::optional<std_filesystem::path> find_near_self(std::string_view filename)
 std::string get_pid_exe(const std::string &pid)
 {
   std::error_code ec;
-  std_filesystem::path proc_path{ "/proc" };
+  std::filesystem::path proc_path{ "/proc" };
   proc_path /= pid;
   proc_path /= "exe";
 
   try {
-    return std_filesystem::read_symlink(proc_path).string();
-  } catch (const std_filesystem::filesystem_error &e) {
+    return std::filesystem::read_symlink(proc_path).string();
+  } catch (const std::filesystem::filesystem_error &e) {
     auto err = e.code().value();
     if (err == ENOENT || err == EINVAL)
       return {};
@@ -314,11 +383,11 @@ std::string get_pid_exe(pid_t pid)
 std::string get_proc_maps(const std::string &pid)
 {
   std::error_code ec;
-  std_filesystem::path proc_path{ "/proc" };
+  std::filesystem::path proc_path{ "/proc" };
   proc_path /= pid;
   proc_path /= "maps";
 
-  if (!std_filesystem::exists(proc_path, ec))
+  if (!std::filesystem::exists(proc_path, ec))
     return "";
 
   return proc_path.string();
@@ -428,8 +497,31 @@ std::string erase_prefix(std::string &str)
   return prefix;
 }
 
-bool wildcard_match(const std::string &str,
-                    std::vector<std::string> &tokens,
+void erase_parameter_list(std::string &demangled_name)
+{
+  size_t args_start = std::string::npos;
+  ssize_t stack = 0;
+  // Look for the parenthesis closing the parameter list, then find
+  // the matching parenthesis at the start of the parameter list...
+  for (ssize_t it = demangled_name.find_last_of(')'); it >= 0; --it) {
+    if (demangled_name[it] == ')')
+      stack++;
+    if (demangled_name[it] == '(')
+      stack--;
+    if (stack == 0) {
+      args_start = it;
+      break;
+    }
+  }
+
+  // If we found the start of the parameter list,
+  // remove the parameters from the match line.
+  if (args_start != std::string::npos)
+    demangled_name.resize(args_start);
+}
+
+bool wildcard_match(std::string_view str,
+                    const std::vector<std::string> &tokens,
                     bool start_wildcard,
                     bool end_wildcard)
 {
@@ -439,7 +531,7 @@ bool wildcard_match(const std::string &str,
     if (str.find(tokens[0], next) != next)
       return false;
 
-  for (std::string token : tokens) {
+  for (const std::string &token : tokens) {
     size_t found = str.find(token, next);
     if (found == std::string::npos)
       return false;
@@ -454,10 +546,8 @@ bool wildcard_match(const std::string &str,
   return true;
 }
 
-/*
- * Splits input string by '*' delimiter and return the individual parts.
- * Sets start_wildcard and end_wildcard if input starts or ends with '*'.
- */
+// Splits input string by '*' delimiter and return the individual parts.
+// Sets start_wildcard and end_wildcard if input starts or ends with '*'.
 std::vector<std::string> get_wildcard_tokens(const std::string &input,
                                              bool &start_wildcard,
                                              bool &end_wildcard)
@@ -483,6 +573,25 @@ std::vector<int> get_possible_cpus()
   return read_cpu_range("/sys/devices/system/cpu/possible");
 }
 
+int get_max_cpu_id()
+{
+  // When booting, the kernel ensures CPUs are ordered from 0 -> N so there
+  // are no gaps in possible CPUs. CPU ID is also u32 so this cast is safe
+  const auto num_possible_cpus = static_cast<uint32_t>(
+      get_possible_cpus().size());
+  assert(num_possible_cpus > 0);
+  // Using global scratch variables for big string usage looks like:
+  //   bounded_cpu_id = bpf_get_smp_processor_id() & MAX_CPU_ID
+  //   buf = global_var[bounded_cpu_id][slot_id]
+  // We bound CPU ID to satisfy the BPF verifier on older kernels. We use an AND
+  // instruction vs. LLVM umin function to reduce the number of jumps in BPF to
+  // ensure we don't hit the complexity limit of 8192 jumps
+  //
+  // To bound using AND, we need to ensure NUM_POSSIBLE_CPUS is rounded up to
+  // next nearest power of 2.
+  return round_up_to_next_power_of_two(num_possible_cpus) - 1;
+}
+
 std::vector<std::string> get_kernel_cflags(const char *uname_machine,
                                            const std::string &ksrc,
                                            const std::string &kobj,
@@ -588,7 +697,7 @@ std::string get_cgroup_path_in_hierarchy(uint64_t cgroupid,
   }
 
   for (auto &path_iter :
-       std_filesystem::recursive_directory_iterator(base_path)) {
+       std::filesystem::recursive_directory_iterator(base_path)) {
     if (stat(path_iter.path().c_str(), &path_st) < 0)
       return "";
     if (path_st.st_ino == cgroupid) {
@@ -602,17 +711,23 @@ std::string get_cgroup_path_in_hierarchy(uint64_t cgroupid,
   return "";
 }
 
-std::vector<std::pair<std::string, std::string>> get_cgroup_hierarchy_roots()
+std::array<std::vector<std::string>, 2> get_cgroup_hierarchy_roots()
 {
   // Get all cgroup mounts and their type (cgroup/cgroup2) from /proc/mounts
   std::ifstream mounts_file("/proc/mounts");
-  std::vector<std::pair<std::string, std::string>> result;
+  std::array<std::vector<std::string>, 2> result;
 
   const std::regex cgroup_mount_regex("(cgroup[2]?) (\\S*)[ ]?.*");
   for (std::string line; std::getline(mounts_file, line);) {
     std::smatch match;
     if (std::regex_match(line, match, cgroup_mount_regex)) {
-      result.push_back({ match[1].str(), match[2].str() });
+      if (std::filesystem::is_directory(match[2].str())) {
+        if (match[1].str() == "cgroup") {
+          result[0].push_back(match[2].str());
+        } else if (match[1].str() == "cgroup2") {
+          result[1].push_back(match[2].str());
+        }
+      }
     }
   }
 
@@ -624,55 +739,49 @@ std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
     uint64_t cgroupid,
     std::string filter)
 {
-  // TODO: Rewrite using std::views when C++20 support becomes common
   auto roots = get_cgroup_hierarchy_roots();
 
   // Replace cgroup version with cgroup mount point directory name for cgroupv1
   // roots and "unified" for cgroupv2 roots
-  for (auto &root : roots) {
-    if (root.first == "cgroup") {
-      root = { std_filesystem::path(root.second).filename().string(),
-               root.second };
-    } else if (root.first == "cgroup2") {
-      root = { "unified", root.second };
-    }
-  }
+  auto types_v1 = roots[0] |
+                  std::views::transform(
+                      [&](auto &root) -> std::pair<std::string, std::string> {
+                        return {
+                          std::filesystem::path(root).filename().string(), root
+                        };
+                      });
+  auto types_v2 = roots[1] |
+                  std::views::transform(
+                      [&](auto &root) -> std::pair<std::string, std::string> {
+                        return { "unified", root };
+                      });
 
   // Filter roots
   bool start_wildcard, end_wildcard;
   auto tokens = get_wildcard_tokens(filter, start_wildcard, end_wildcard);
-  std::vector<std::pair<std::string, std::string>> filtered_roots;
-  std::copy_if(roots.begin(),
-               roots.end(),
-               std::back_inserter(filtered_roots),
-               [&tokens, &start_wildcard, &end_wildcard](auto &pair) {
-                 return wildcard_match(
-                     pair.first, tokens, start_wildcard, end_wildcard);
-               });
+  auto filter_func = std::views::filter([&](auto pair) -> bool {
+    return wildcard_match(pair.first, tokens, start_wildcard, end_wildcard);
+  });
+  auto filtered_v1 = types_v1 | filter_func;
+  auto filtered_v2 = types_v2 | filter_func;
 
   // Get cgroup path for each root
-  std::vector<std::pair<std::string, std::string>> result;
-  std::transform(filtered_roots.begin(),
-                 filtered_roots.end(),
-                 std::back_inserter(result),
-                 [&cgroupid](auto &pair) {
-                   return std::pair<std::string, std::string>{
-                     pair.first,
-                     get_cgroup_path_in_hierarchy(cgroupid, pair.second)
-                   };
-                 });
+  auto get_path_func = std::views::transform(
+      [&](auto pair) -> std::pair<std::string, std::string> {
+        return { pair.first,
+                 get_cgroup_path_in_hierarchy(cgroupid, pair.second) };
+      });
+  auto paths_v1 = filtered_v1 | get_path_func;
+  auto paths_v2 = filtered_v2 | get_path_func;
 
-  // Sort paths lexically by name (with the exception of unified, which always
-  // comes first)
-  std::sort(result.begin(), result.end(), [](auto &pair1, auto &pair2) {
-    if (pair2.first == "unified")
-      return false;
-    if (pair1.first == "unified")
-      return true;
-    return pair1.first < pair2.first;
-  });
-
-  return result;
+  // Return paths with v2 first, then v1 sorted lexically by name.
+  std::vector<std::pair<std::string, std::string>> sorted(paths_v2.begin(),
+                                                          paths_v2.end());
+  std::vector<std::pair<std::string, std::string>> sorted_v1(paths_v1.begin(),
+                                                             paths_v1.end());
+  std::sort(sorted_v1.begin(), sorted_v1.end());
+  sorted.insert(sorted.end(), sorted_v1.begin(), sorted_v1.end());
+  return sorted;
 }
 
 bool is_module_loaded(const std::string &module)
@@ -698,12 +807,12 @@ bool is_module_loaded(const std::string &module)
 bool is_dir(const std::string &path)
 {
   std::error_code ec;
-  std_filesystem::path buf{ path };
-  return std_filesystem::is_directory(buf, ec);
+  std::filesystem::path buf{ path };
+  return std::filesystem::is_directory(buf, ec);
 }
 
-// get_kernel_dirs returns {ksrc, kobj} - directories for pristine and
-// generated kernel sources.
+// get_kernel_dirs fills {ksrc, kobj} - directories for pristine and
+// generated kernel sources - and returns if they were found.
 //
 // When the kernel was built in its source tree ksrc == kobj, however when
 // the kernel was build in a different directory than its source, ksrc != kobj.
@@ -716,43 +825,40 @@ bool is_dir(const std::string &path)
 //
 //   /lib/modules/`uname -r`/build/
 //
-// {"", ""} is returned if no trace of kernel headers was found at all.
-// Both ksrc and kobj are guaranteed to be != "", if at least some trace of
-// kernel sources was found.
-std::tuple<std::string, std::string> get_kernel_dirs(
-    const struct utsname &utsname)
+// false is returned if no trace of kernel headers was found at all, with the
+// guessed location set anyway for later warning.
+//
+// Both ksrc and kobj are guaranteed to be != ""
+bool get_kernel_dirs(const struct utsname &utsname,
+                     std::string &ksrc,
+                     std::string &kobj)
 {
-#ifdef KERNEL_HEADERS_DIR
-  return { KERNEL_HEADERS_DIR, KERNEL_HEADERS_DIR };
-#endif
+  ksrc = kobj = std::string(KERNEL_HEADERS_DIR);
+  if (!ksrc.empty())
+    return true;
 
   const char *kpath_env = ::getenv("BPFTRACE_KERNEL_SOURCE");
   if (kpath_env) {
+    ksrc = std::string(kpath_env);
     const char *kpath_build_env = ::getenv("BPFTRACE_KERNEL_BUILD");
-    if (!kpath_build_env) {
-      kpath_build_env = kpath_env;
+    if (kpath_build_env) {
+      kobj = std::string(kpath_build_env);
+    } else {
+      kobj = ksrc;
     }
-    return std::make_tuple(kpath_env, kpath_build_env);
+    return true;
   }
 
   std::string kdir = std::string("/lib/modules/") + utsname.release;
-  auto ksrc = kdir + "/source";
-  auto kobj = kdir + "/build";
+  ksrc = kdir + "/source";
+  kobj = kdir + "/build";
 
   // if one of source/ or build/ is not present - try to use the other one for
   // both.
   auto has_ksrc = is_dir(ksrc);
   auto has_kobj = is_dir(kobj);
   if (!has_ksrc && !has_kobj) {
-    LOG(WARNING) << "Could not find kernel headers in " << ksrc << " or "
-                 << kobj
-                 << ". To specify a particular path to kernel headers, set the "
-                    "env variables BPFTRACE_KERNEL_SOURCE and, optionally, "
-                    "BPFTRACE_KERNEL_BUILD if the kernel was built in a "
-                    "different directory than its source. To create kernel "
-                    "headers run 'modprobe kheaders', which will create a tar "
-                    "file at /sys/kernel/kheaders.tar.xz";
-    return std::make_tuple("", "");
+    return false;
   }
   if (!has_ksrc) {
     ksrc = kobj;
@@ -760,7 +866,7 @@ std::tuple<std::string, std::string> get_kernel_dirs(
     kobj = ksrc;
   }
 
-  return std::make_tuple(ksrc, kobj);
+  return true;
 }
 
 const std::string &is_deprecated(const std::string &str)
@@ -828,20 +934,17 @@ std::string exec_system(const char *cmd)
   return result;
 }
 
-/*
-Original resolve_binary_path API defaulting to bpftrace's mount namespace
-*/
+// Original resolve_binary_path API defaulting to bpftrace's mount namespace
 std::vector<std::string> resolve_binary_path(const std::string &cmd)
 {
   const char *env_paths = getenv("PATH");
   return resolve_binary_path(cmd, env_paths, -1);
 }
 
-/*
-If a pid is specified, the binary path is taken relative to its own PATH if
-it is in a different mount namespace. Otherwise, the path is resolved relative
-to the local PATH env var for bpftrace's own mount namespace if it is set
-*/
+// If a pid is specified, the binary path is taken relative to its own PATH if
+// it is in a different mount namespace. Otherwise, the path is resolved
+// relative to the local PATH env var for bpftrace's own mount namespace if it
+// is set
 std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid)
 {
   std::string env_paths = "";
@@ -867,10 +970,68 @@ std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid)
   }
 }
 
-/*
-Private interface to resolve_binary_path, used for the exposed variants above,
-allowing for a PID whose mount namespace should be optionally considered.
-*/
+// Check whether 'path' refers to a ELF file. Errors are swallowed silently and
+// result in return of 'nullopt'. On success, the ELF type (e.g., ET_DYN) is
+// returned.
+static std::optional<int> is_elf(const std::string &path)
+{
+  int fd;
+  Elf *elf;
+  GElf_Ehdr ehdr;
+
+  if (elf_version(EV_CURRENT) == EV_NONE) {
+    return std::nullopt;
+  }
+
+  fd = open(path.c_str(), O_RDONLY, 0);
+  if (fd < 0) {
+    return std::nullopt;
+  }
+  SCOPE_EXIT
+  {
+    ::close(fd);
+  };
+
+  elf = elf_begin(fd, ELF_C_READ, nullptr);
+  if (elf == nullptr) {
+    return std::nullopt;
+  }
+  SCOPE_EXIT
+  {
+    ::elf_end(elf);
+  };
+
+  if (elf_kind(elf) != ELF_K_ELF) {
+    return std::nullopt;
+  }
+
+  if (!gelf_getehdr(elf, &ehdr)) {
+    return std::nullopt;
+  }
+
+  return ehdr.e_type;
+}
+
+static bool has_exec_permission(const std::string &path)
+{
+  using std::filesystem::perms;
+
+  auto perms = std::filesystem::status(path).permissions();
+  return (perms & perms::owner_exec) != perms::none;
+}
+
+// Check whether 'path' refers to an executable ELF file.
+bool is_exe(const std::string &path)
+{
+  if (auto e_type = is_elf(path)) {
+    return e_type == ET_EXEC && has_exec_permission(path);
+  }
+  return false;
+}
+
+// Private interface to resolve_binary_path, used for the exposed variants
+// above, allowing for a PID whose mount namespace should be optionally
+// considered.
 static std::vector<std::string> resolve_binary_path(const std::string &cmd,
                                                     const char *env_paths,
                                                     int pid)
@@ -891,9 +1052,14 @@ static std::vector<std::string> resolve_binary_path(const std::string &cmd,
       rel_path = path_for_pid_mountns(pid, path);
     else
       rel_path = path;
-    if (bcc_elf_is_exe(rel_path.c_str()) ||
-        bcc_elf_is_shared_obj(rel_path.c_str()))
-      valid_executable_paths.push_back(rel_path);
+
+    // Both executables and shared objects are game.
+    if (auto e_type = is_elf(rel_path)) {
+      if ((e_type == ET_EXEC && has_exec_permission(rel_path)) ||
+          e_type == ET_DYN) {
+        valid_executable_paths.push_back(rel_path);
+      }
+    }
   }
 
   return valid_executable_paths;
@@ -916,43 +1082,41 @@ std::string path_for_pid_mountns(int pid, const std::string &path)
   return pid_relative_path.str();
 }
 
-/*
-Determines if the target process is in a different mount namespace from
-bpftrace.
-
-If a process is in a different mount namespace (eg, container) it is very
-likely that any references to local paths will not be valid, and that paths
-need to be made relative to the PID.
-
-If an invalid PID is specified or doesn't exist, it returns false.
-True is only returned if the namespace of the target process could be read and
-it doesn't match that of bpftrace. If there was an error reading either mount
-namespace, it will throw an exception
-*/
+// Determines if the target process is in a different mount namespace from
+// bpftrace.
+//
+// If a process is in a different mount namespace (eg, container) it is very
+// likely that any references to local paths will not be valid, and that paths
+// need to be made relative to the PID.
+//
+// If an invalid PID is specified or doesn't exist, it returns false.
+// True is only returned if the namespace of the target process could be read
+// and it doesn't match that of bpftrace. If there was an error reading either
+// mount namespace, it will throw an exception
 static bool pid_in_different_mountns(int pid)
 {
   if (pid <= 0)
     return false;
 
   std::error_code ec;
-  std_filesystem::path self_path{ "/proc/self/ns/mnt" };
-  std_filesystem::path target_path{ "/proc" };
+  std::filesystem::path self_path{ "/proc/self/ns/mnt" };
+  std::filesystem::path target_path{ "/proc" };
   target_path /= std::to_string(pid);
   target_path /= "ns/mnt";
 
-  if (!std_filesystem::exists(self_path, ec)) {
+  if (!std::filesystem::exists(self_path, ec)) {
     throw MountNSException(
         "Failed to compare mount ns with PID " + std::to_string(pid) +
         ". The error was open (/proc/self/ns/mnt): " + ec.message());
   }
 
-  if (!std_filesystem::exists(target_path, ec)) {
+  if (!std::filesystem::exists(target_path, ec)) {
     throw MountNSException(
         "Failed to compare mount ns with PID " + std::to_string(pid) +
         ". The error was open (/proc/<pid>/ns/mnt): " + ec.message());
   }
 
-  bool result = !std_filesystem::equivalent(self_path, target_path, ec);
+  bool result = !std::filesystem::equivalent(self_path, target_path, ec);
 
   if (ec) {
     throw MountNSException("Failed to compare mount ns with PID " +
@@ -1010,15 +1174,47 @@ std::string str_join(const std::vector<std::string> &list,
   return str;
 }
 
-bool is_numeric(const std::string &s)
+std::optional<std::variant<int64_t, uint64_t>> get_int_from_str(
+    const std::string &s)
 {
-  std::size_t idx;
-  try {
-    std::stoll(s, &idx, 0);
-  } catch (...) {
-    return false;
+  if (s.size() == 0) {
+    return std::nullopt;
   }
-  return idx == s.size();
+
+  if (s.starts_with("0x") || s.starts_with("0X")) {
+    // Treat all hex's as unsigned
+    std::size_t idx;
+    try {
+      uint64_t ret = std::stoull(s, &idx, 0);
+      if (idx == s.size()) {
+        return ret;
+      } else {
+        return std::nullopt;
+      }
+    } catch (...) {
+      return std::nullopt;
+    }
+  }
+
+  char *endptr;
+  const char *s_ptr = s.c_str();
+  errno = 0;
+
+  if (s.at(0) == '-') {
+    int64_t ret = strtol(s_ptr, &endptr, 0);
+    if (endptr == s_ptr || *endptr != '\0' || errno == ERANGE ||
+        errno == EINVAL) {
+      return std::nullopt;
+    }
+    return ret;
+  }
+
+  uint64_t ret = strtoul(s_ptr, &endptr, 0);
+  if (endptr == s_ptr || *endptr != '\0' || errno == ERANGE ||
+      errno == EINVAL) {
+    return std::nullopt;
+  }
+  return ret;
 }
 
 bool symbol_has_cpp_mangled_signature(const std::string &sym_name)
@@ -1071,30 +1267,33 @@ std::string hex_format_buffer(const char *buf,
                               bool escape_hex)
 {
   // Allow enough space for every byte to be sanitized in the form "\x00"
-  char s[size * 4 + 1];
+  std::string str(size * 4 + 1, '\0');
+  char *s = str.data();
 
   size_t offset = 0;
   for (size_t i = 0; i < size; i++)
     if (keep_ascii && buf[i] >= 32 && buf[i] <= 126)
-      offset += sprintf(s + offset, "%c", ((const uint8_t *)buf)[i]);
+      offset += sprintf(s + offset,
+                        "%c",
+                        (reinterpret_cast<const uint8_t *>(buf))[i]);
     else if (escape_hex)
-      offset += sprintf(s + offset, "\\x%02x", ((const uint8_t *)buf)[i]);
+      offset += sprintf(s + offset,
+                        "\\x%02x",
+                        (reinterpret_cast<const uint8_t *>(buf))[i]);
     else
       offset += sprintf(s + offset,
                         i == size - 1 ? "%02x" : "%02x ",
-                        ((const uint8_t *)buf)[i]);
+                        (reinterpret_cast<const uint8_t *>(buf))[i]);
 
-  s[offset] = '\0';
-
-  return std::string(s);
+  // Fit return value to actual length
+  str.resize(offset);
+  return str;
 }
 
-/*
- * Attaching to these kernel functions with kfunc/fentry or kretfunc/fexit
- * could lead to a recursive loop and kernel crash so we need additional
- * generated BPF code to protect against this if one of these are being
- * attached to.
- */
+// Attaching to these kernel functions with fentry/fexit (kfunc/kretfunc)
+// could lead to a recursive loop and kernel crash so we need additional
+// generated BPF code to protect against this if one of these are being
+// attached to.
 bool is_recursive_func(const std::string &func_name)
 {
   return RECURSIVE_KERNEL_FUNCS.find(func_name) != RECURSIVE_KERNEL_FUNCS.end();
@@ -1102,12 +1301,10 @@ bool is_recursive_func(const std::string &func_name)
 
 static bool is_bad_func(std::string &func)
 {
-  /*
-   * Certain kernel functions are known to cause system stability issues if
-   * traced (but not marked "notrace" in the kernel) so they should be filtered
-   * out as the list is built. The list of functions have been taken from the
-   * bpf kernel selftests (bpf/prog_tests/kprobe_multi_test.c).
-   */
+  // Certain kernel functions are known to cause system stability issues if
+  // traced (but not marked "notrace" in the kernel) so they should be filtered
+  // out as the list is built. The list of functions have been taken from the
+  // bpf kernel selftests (bpf/prog_tests/kprobe_multi_test.c).
   static const std::unordered_set<std::string> bad_funcs = {
     "arch_cpu_idle", "default_idle", "bpf_dispatcher_xdp_func"
   };
@@ -1170,9 +1367,7 @@ FuncsModulesMap parse_traceable_funcs()
 #endif
 }
 
-/**
- * Search for LINUX_VERSION_CODE in the vDSO, returning 0 if it can't be found.
- */
+// Search for LINUX_VERSION_CODE in the vDSO, returning 0 if it can't be found.
 static uint32_t _find_version_note(unsigned long base)
 {
   auto ehdr = reinterpret_cast<const ElfW(Ehdr) *>(base);
@@ -1205,7 +1400,7 @@ static uint32_t _find_version_note(unsigned long base)
   return 0;
 }
 
-static uint32_t kernel_version_from_vdso(void)
+static uint32_t kernel_version_from_vdso()
 {
   // Fetch LINUX_VERSION_CODE from the vDSO .note section, falling back on
   // the build-time constant if unavailable. This always matches the
@@ -1219,7 +1414,7 @@ static uint32_t kernel_version_from_vdso(void)
   return code;
 }
 
-static uint32_t kernel_version_from_uts(void)
+static uint32_t kernel_version_from_uts()
 {
   struct utsname utsname;
   if (uname(&utsname) < 0)
@@ -1230,14 +1425,14 @@ static uint32_t kernel_version_from_uts(void)
   return KERNEL_VERSION(x, y, z);
 }
 
-static uint32_t kernel_version_from_khdr(void)
+static uint32_t kernel_version_from_khdr()
 {
   // Try to get the definition of LINUX_VERSION_CODE at runtime.
   std::ifstream linux_version_header{ "/usr/include/linux/version.h" };
   const std::string content{ std::istreambuf_iterator<char>(
                                  linux_version_header),
                              std::istreambuf_iterator<char>() };
-  const std::regex regex{ "#define\\s+LINUX_VERSION_CODE\\s+(\\d+)" };
+  const std::regex regex{ R"(#define\s+LINUX_VERSION_CODE\s+(\d+))" };
   std::smatch match;
 
   if (std::regex_search(content.begin(), content.end(), match, regex))
@@ -1246,11 +1441,9 @@ static uint32_t kernel_version_from_khdr(void)
   return 0;
 }
 
-/**
- * Find a LINUX_VERSION_CODE matching the host kernel. The build-time constant
- * may not match if bpftrace is compiled on a different Linux version than it's
- * used on, e.g. if built with Docker.
- */
+// Find a LINUX_VERSION_CODE matching the host kernel. The build-time constant
+// may not match if bpftrace is compiled on a different Linux version than it's
+// used on, e.g. if built with Docker.
 uint32_t kernel_version(KernelVersionMethod method)
 {
   static std::optional<uint32_t> a0, a1, a2;
@@ -1286,9 +1479,9 @@ std::optional<std::string> abs_path(const std::string &rel_path)
   static auto re = std::regex("^/proc/\\d+/root/.*");
   if (!std::regex_match(rel_path, re)) {
     try {
-      auto p = std_filesystem::path(rel_path);
-      return std_filesystem::canonical(std_filesystem::absolute(p)).string();
-    } catch (std_filesystem::filesystem_error &) {
+      auto p = std::filesystem::path(rel_path);
+      return std::filesystem::canonical(std::filesystem::absolute(p)).string();
+    } catch (std::filesystem::filesystem_error &) {
       return {};
     }
   } else {
@@ -1370,7 +1563,7 @@ std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
 std::vector<int> get_pids_for_program(const std::string &program)
 {
   std::error_code ec;
-  auto program_abs = std_filesystem::canonical(program, ec);
+  auto program_abs = std::filesystem::canonical(program, ec);
   if (ec) {
     // std::filesystem::canonical will fail if we are attaching to a uprobe that
     // lives in another filesystem namespace. For example,
@@ -1383,12 +1576,12 @@ std::vector<int> get_pids_for_program(const std::string &program)
   }
 
   std::vector<int> pids;
-  for (const auto &process : std_filesystem::directory_iterator("/proc")) {
+  for (const auto &process : std::filesystem::directory_iterator("/proc")) {
     std::string filename = process.path().filename().string();
     if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
       continue;
     std::error_code ec;
-    std_filesystem::path pid_program = std_filesystem::read_symlink(
+    std::filesystem::path pid_program = std::filesystem::read_symlink(
         process.path() / "exe", ec);
     if (!ec && program_abs == pid_program)
       pids.emplace_back(std::stoi(filename));
@@ -1399,7 +1592,7 @@ std::vector<int> get_pids_for_program(const std::string &program)
 std::vector<int> get_all_running_pids()
 {
   std::vector<int> pids;
-  for (const auto &process : std_filesystem::directory_iterator("/proc")) {
+  for (const auto &process : std::filesystem::directory_iterator("/proc")) {
     std::string filename = process.path().filename().string();
     if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
       continue;
@@ -1433,4 +1626,23 @@ std::string sanitise_bpf_program_name(const std::string &name)
   return sanitised_name;
 }
 
+uint32_t round_up_to_next_power_of_two(uint32_t n)
+{
+  // http://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
+  if (n == 0) {
+    return 0;
+  }
+  // Note this function doesn't work if n > 2^31 since there are not enough
+  // bits in unsigned 32 bit integers. This should be fine since its unlikely
+  // anyone has > 2^31 CPUs.
+  assert(n <= 2147483648);
+  n--;
+  n |= n >> 1;
+  n |= n >> 2;
+  n |= n >> 4;
+  n |= n >> 8;
+  n |= n >> 16;
+  return n + 1;
+}
+
 } // namespace bpftrace
