@@ -3,6 +3,7 @@
 #include <cstdint>
 #include <cstring>
 #include <exception>
+#include <filesystem>
 #include <functional>
 #include <iostream>
 #include <map>
@@ -15,9 +16,10 @@
 #include <tuple>
 #include <unordered_map>
 #include <unordered_set>
+#include <variant>
 #include <vector>
 
-#include "filesystem.h"
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
 namespace bpftrace {
 
@@ -170,9 +172,9 @@ void get_uint64_env_var(const ::std::string &str,
 void get_bool_env_var(const ::std::string &str,
                       const std::function<void(bool)> &cb);
 // Tries to find a file in $PATH
-std::optional<std_filesystem::path> find_in_path(std::string_view name);
+std::optional<std::filesystem::path> find_in_path(std::string_view name);
 // Finds a file in the same directory as running binary
-std::optional<std_filesystem::path> find_near_self(std::string_view name);
+std::optional<std::filesystem::path> find_near_self(std::string_view name);
 std::string get_pid_exe(pid_t pid);
 std::string get_pid_exe(const std::string &pid);
 std::string get_proc_maps(const std::string &pid);
@@ -182,8 +184,9 @@ std::vector<std::string> split_string(const std::string &str,
                                       char delimiter,
                                       bool remove_empty = false);
 std::string erase_prefix(std::string &str);
-bool wildcard_match(const std::string &str,
-                    std::vector<std::string> &tokens,
+void erase_parameter_list(std::string &demangled_name);
+bool wildcard_match(std::string_view str,
+                    const std::vector<std::string> &tokens,
                     bool start_wildcard,
                     bool end_wildcard);
 std::vector<std::string> get_wildcard_tokens(const std::string &input,
@@ -191,17 +194,20 @@ std::vector<std::string> get_wildcard_tokens(const std::string &input,
                                              bool &end_wildcard);
 std::vector<int> get_online_cpus();
 std::vector<int> get_possible_cpus();
+int get_max_cpu_id();
 bool is_dir(const std::string &path);
 bool file_exists_and_ownedby_root(const char *f);
-std::tuple<std::string, std::string> get_kernel_dirs(
-    const struct utsname &utsname);
+std::optional<std::string> find_vmlinux(struct symbol *sym = nullptr);
+bool get_kernel_dirs(const struct utsname &utsname,
+                     std::string &ksrc,
+                     std::string &kobj);
 std::vector<std::string> get_kernel_cflags(const char *uname_machine,
                                            const std::string &ksrc,
                                            const std::string &kobj,
                                            const KConfig &kconfig);
 std::string get_cgroup_path_in_hierarchy(uint64_t cgroupid,
                                          std::string base_path);
-std::vector<std::pair<std::string, std::string>> get_cgroup_hierarchy_roots();
+std::array<std::vector<std::string>, 2> get_cgroup_hierarchy_roots();
 std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
     uint64_t cgroupid,
     std::string filter);
@@ -214,13 +220,15 @@ bool is_compile_time_func(const std::string &func_name);
 bool is_supported_lang(const std::string &lang);
 bool is_type_name(std::string_view str);
 std::string exec_system(const char *cmd);
+bool is_exe(const std::string &path);
 std::vector<std::string> resolve_binary_path(const std::string &cmd);
 std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid);
 std::string path_for_pid_mountns(int pid, const std::string &path);
 void cat_file(const char *filename, size_t, std::ostream &);
 std::string str_join(const std::vector<std::string> &list,
                      const std::string &delim);
-bool is_numeric(const std::string &str);
+std::optional<std::variant<int64_t, uint64_t>> get_int_from_str(
+    const std::string &s);
 bool symbol_has_cpp_mangled_signature(const std::string &sym_name);
 std::optional<pid_t> parse_pid(const std::string &str, std::string &err);
 std::string hex_format_buffer(const char *buf,
@@ -250,6 +258,8 @@ std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
 std::vector<int> get_pids_for_program(const std::string &program);
 std::vector<int> get_all_running_pids();
 
+uint32_t round_up_to_next_power_of_two(uint32_t n);
+
 std::string sanitise_bpf_program_name(const std::string &name);
 // Generate object file function name for a given probe
 inline std::string get_function_name_for_probe(
@@ -351,6 +361,33 @@ T min_max_value(const std::vector<uint8_t> &value, int nvalues, bool is_max)
   return mm_val;
 }
 
+template <typename T>
+struct stats {
+  T total;
+  T count;
+  T avg;
+};
+
+template <typename T>
+stats<T> stats_value(const std::vector<uint8_t> &value, int nvalues)
+{
+  stats<T> ret = { 0, 0, 0 };
+  for (int i = 0; i < nvalues; i++) {
+    T val = read_data<T>(value.data() + i * (sizeof(T) * 2));
+    T cpu_count = read_data<T>(value.data() + sizeof(T) + i * (sizeof(T) * 2));
+    ret.count += cpu_count;
+    ret.total += val;
+  }
+  ret.avg = (T)(ret.total / ret.count);
+  return ret;
+}
+
+template <typename T>
+T avg_value(const std::vector<uint8_t> &value, int nvalues)
+{
+  return stats_value<T>(value, nvalues).avg;
+}
+
 // Combination of 2 hashes
 // The algorithm is taken from boost::hash_combine
 template <class T>
@@ -360,4 +397,47 @@ inline void hash_combine(std::size_t &seed, const T &value)
   seed ^= hasher(value) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
 }
 
+struct symbol {
+  std::string name;
+  uint64_t start;
+  uint64_t size;
+  uint64_t address;
+};
+
+inline int sym_name_cb(const char *symname,
+                       uint64_t start,
+                       uint64_t size,
+                       void *p)
+{
+  struct symbol *sym = static_cast<struct symbol *>(p);
+
+  if (sym->name == symname) {
+    sym->start = start;
+    sym->size = size;
+    return -1;
+  }
+
+  return 0;
+}
+
+inline int sym_address_cb(const char *symname,
+                          uint64_t start,
+                          uint64_t size,
+                          void *p)
+{
+  struct symbol *sym = static_cast<struct symbol *>(p);
+
+  // When size is 0, then [start, start + size) = [start, start) = ø.
+  // So we need a special case when size=0, but address matches the symbol's
+  if (sym->address == start ||
+      (sym->address > start && sym->address < (start + size))) {
+    sym->start = start;
+    sym->size = size;
+    sym->name = symname;
+    return -1;
+  }
+
+  return 0;
+}
+
 } // namespace bpftrace
