@@ -1,15 +1,16 @@
 #include "clang_parser.h"
+
+#include <iostream>
+#include <utility>
+
 #include "ast/passes/field_analyser.h"
 #include "bpftrace.h"
 #include "driver.h"
 #include "struct.h"
 #include "gtest/gtest.h"
-#include <iostream>
 #include <llvm/Config/llvm-config.h>
 
-namespace bpftrace {
-namespace test {
-namespace clang_parser {
+namespace bpftrace::test::clang_parser {
 
 #include "btf_common.h"
 
@@ -22,11 +23,11 @@ static void parse(const std::string &input,
   Driver driver(bpftrace);
   ASSERT_EQ(driver.parse_str(extended_input), 0);
 
-  ast::FieldAnalyser fields(driver.root.get(), bpftrace);
+  ast::FieldAnalyser fields(driver.ctx, bpftrace);
   EXPECT_EQ(fields.analyse(), 0);
 
   ClangParser clang;
-  ASSERT_EQ(clang.parse(driver.root.get(), bpftrace), result);
+  ASSERT_EQ(clang.parse(driver.ctx.root, bpftrace), result);
 }
 
 TEST(clang_parser, integers)
@@ -91,7 +92,7 @@ TEST(clang_parser, c_union)
 TEST(clang_parser, c_enum)
 {
   BPFtrace bpftrace;
-  parse("enum E {NONE}; struct Foo { enum E e; }", bpftrace);
+  parse("enum E { NONE, SOME = 99, }; struct Foo { enum E e; }", bpftrace);
 
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo"));
   auto foo = bpftrace.structs.Lookup("struct Foo").lock();
@@ -103,6 +104,92 @@ TEST(clang_parser, c_enum)
   EXPECT_TRUE(foo->GetField("e").type.IsIntTy());
   EXPECT_EQ(foo->GetField("e").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("e").offset, 0);
+
+  ASSERT_TRUE(bpftrace.enums_.contains("NONE"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["NONE"]), 0);
+  EXPECT_EQ(std::get<1>(bpftrace.enums_["NONE"]), "E");
+  ASSERT_TRUE(bpftrace.enums_.contains("SOME"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["SOME"]), 99);
+  EXPECT_EQ(std::get<1>(bpftrace.enums_["SOME"]), "E");
+
+  ASSERT_TRUE(bpftrace.enum_defs_.contains("E"));
+  ASSERT_TRUE(bpftrace.enum_defs_["E"].contains(0));
+  EXPECT_EQ(bpftrace.enum_defs_["E"][0], "NONE");
+  ASSERT_TRUE(bpftrace.enum_defs_["E"].contains(99));
+  EXPECT_EQ(bpftrace.enum_defs_["E"][99], "SOME");
+}
+
+TEST(clang_parser, c_enum_anonymous)
+{
+  BPFtrace bpftrace;
+  parse(
+      "enum { ANON_A_VARIANT_1 = 0, ANON_A_VARIANT_2, ANON_A_CONFLICT = 99, }; "
+      "enum { ANON_B_VARIANT_1 = 0, ANON_B_CONFLICT = 99, }; ",
+      bpftrace);
+
+  ASSERT_EQ(bpftrace.enums_.size(), 5);
+  ASSERT_EQ(bpftrace.enum_defs_.size(), 2);
+
+  //
+  // Check enums_ contains first anonymous enum
+  //
+
+  // Check first variant present
+  ASSERT_TRUE(bpftrace.enums_.contains("ANON_A_VARIANT_1"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["ANON_A_VARIANT_1"]), 0);
+  auto anon_a_name = std::get<1>(bpftrace.enums_["ANON_A_VARIANT_1"]);
+  ASSERT_FALSE(anon_a_name.empty());
+
+  // Check second variant present
+  ASSERT_TRUE(bpftrace.enums_.contains("ANON_A_VARIANT_2"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["ANON_A_VARIANT_2"]), 1);
+  EXPECT_EQ(std::get<1>(bpftrace.enums_["ANON_A_CONFLICT"]), anon_a_name);
+
+  // Check conflict variant present
+  ASSERT_TRUE(bpftrace.enums_.contains("ANON_A_CONFLICT"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["ANON_A_CONFLICT"]), 99);
+  EXPECT_EQ(std::get<1>(bpftrace.enums_["ANON_A_CONFLICT"]), anon_a_name);
+
+  //
+  // Check enum_defs_ contains first anonymous enum, with ANON_A_CONFLICT
+  // value resolving correctly to the this enum and not the other.
+  //
+
+  ASSERT_TRUE(bpftrace.enum_defs_.contains(anon_a_name));
+  ASSERT_EQ(bpftrace.enum_defs_[anon_a_name].size(), 3);
+  ASSERT_TRUE(bpftrace.enum_defs_[anon_a_name].contains(0));
+  EXPECT_EQ(bpftrace.enum_defs_[anon_a_name][0], "ANON_A_VARIANT_1");
+  ASSERT_TRUE(bpftrace.enum_defs_[anon_a_name].contains(1));
+  EXPECT_EQ(bpftrace.enum_defs_[anon_a_name][1], "ANON_A_VARIANT_2");
+  ASSERT_TRUE(bpftrace.enum_defs_[anon_a_name].contains(99));
+  EXPECT_EQ(bpftrace.enum_defs_[anon_a_name][99], "ANON_A_CONFLICT");
+
+  //
+  // Check enums_ contains second anonymous enum
+  //
+
+  // Check first variant present
+  ASSERT_TRUE(bpftrace.enums_.contains("ANON_B_VARIANT_1"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["ANON_B_VARIANT_1"]), 0);
+  auto anon_b_name = std::get<1>(bpftrace.enums_["ANON_B_VARIANT_1"]);
+  ASSERT_FALSE(anon_b_name.empty());
+
+  // Check conflict variant present
+  ASSERT_TRUE(bpftrace.enums_.contains("ANON_B_CONFLICT"));
+  EXPECT_EQ(std::get<0>(bpftrace.enums_["ANON_B_CONFLICT"]), 99);
+  EXPECT_EQ(std::get<1>(bpftrace.enums_["ANON_B_CONFLICT"]), anon_b_name);
+
+  //
+  // Check enum_defs_ contains second anonymous enum, with ANON_B_CONFLICT
+  // value resolving correctly to the this enum and not the first.
+  //
+
+  ASSERT_TRUE(bpftrace.enum_defs_.contains(anon_b_name));
+  ASSERT_EQ(bpftrace.enum_defs_[anon_b_name].size(), 2);
+  ASSERT_TRUE(bpftrace.enum_defs_[anon_b_name].contains(0));
+  EXPECT_EQ(bpftrace.enum_defs_[anon_b_name][0], "ANON_B_VARIANT_1");
+  ASSERT_TRUE(bpftrace.enum_defs_[anon_b_name].contains(99));
+  EXPECT_EQ(bpftrace.enum_defs_[anon_b_name][99], "ANON_B_CONFLICT");
 }
 
 TEST(clang_parser, integer_ptr)
@@ -208,13 +295,8 @@ TEST(clang_parser, nested_struct_no_type)
   parse("struct Foo { struct { int x; } bar; union { int y; } baz; }",
         bpftrace);
 
-#if LLVM_VERSION_MAJOR >= 13
   std::string bar_name = "struct Foo::(unnamed at definitions.h:2:14)";
   std::string baz_name = "union Foo::(unnamed at definitions.h:2:37)";
-#else
-  std::string bar_name = "struct Foo::(anonymous at definitions.h:2:14)";
-  std::string baz_name = "union Foo::(anonymous at definitions.h:2:37)";
-#endif
 
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo"));
   ASSERT_TRUE(bpftrace.structs.Has(bar_name));
@@ -359,10 +441,14 @@ TEST(clang_parser, bitfields)
   ASSERT_TRUE(foo->HasField("b"));
   ASSERT_TRUE(foo->HasField("c"));
 
+  // clang-tidy doesn't seem to acknowledge that ASSERT_*() will
+  // return from function so that these are in fact checked accesses.
+  //
+  // NOLINTBEGIN(bugprone-unchecked-optional-access)
   EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 0);
-  EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("a").bitfield.has_value());
   EXPECT_EQ(foo->GetField("a").bitfield->read_bytes, 0x1U);
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0xFFU);
@@ -382,6 +468,7 @@ TEST(clang_parser, bitfields)
   EXPECT_EQ(foo->GetField("c").bitfield->read_bytes, 0x2U);
   EXPECT_EQ(foo->GetField("c").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("c").bitfield->mask, 0xFFFFU);
+  // NOLINTEND(bugprone-unchecked-optional-access)
 }
 
 TEST(clang_parser, bitfields_uneven_fields)
@@ -400,10 +487,14 @@ TEST(clang_parser, bitfields_uneven_fields)
   ASSERT_TRUE(foo->HasField("d"));
   ASSERT_TRUE(foo->HasField("e"));
 
+  // clang-tidy doesn't seem to acknowledge that ASSERT_*() will
+  // return from function so that these are in fact checked accesses.
+  //
+  // NOLINTBEGIN(bugprone-unchecked-optional-access)
   EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 0);
-  EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("a").bitfield.has_value());
   EXPECT_EQ(foo->GetField("a").bitfield->read_bytes, 1U);
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0x1U);
@@ -411,7 +502,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_TRUE(foo->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("b").offset, 0);
-  EXPECT_TRUE(foo->GetField("b").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("b").bitfield.has_value());
   EXPECT_EQ(foo->GetField("b").bitfield->read_bytes, 1U);
   EXPECT_EQ(foo->GetField("b").bitfield->access_rshift, 1U);
   EXPECT_EQ(foo->GetField("b").bitfield->mask, 0x1U);
@@ -419,7 +510,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_TRUE(foo->GetField("c").type.IsIntTy());
   EXPECT_EQ(foo->GetField("c").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("c").offset, 0);
-  EXPECT_TRUE(foo->GetField("c").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("c").bitfield.has_value());
   EXPECT_EQ(foo->GetField("c").bitfield->read_bytes, 1U);
   EXPECT_EQ(foo->GetField("c").bitfield->access_rshift, 2U);
   EXPECT_EQ(foo->GetField("c").bitfield->mask, 0x7U);
@@ -427,7 +518,7 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_TRUE(foo->GetField("d").type.IsIntTy());
   EXPECT_EQ(foo->GetField("d").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("d").offset, 0);
-  EXPECT_TRUE(foo->GetField("d").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("d").bitfield.has_value());
   EXPECT_EQ(foo->GetField("d").bitfield->read_bytes, 4U);
   EXPECT_EQ(foo->GetField("d").bitfield->access_rshift, 5U);
   EXPECT_EQ(foo->GetField("d").bitfield->mask, 0xFFFFFU);
@@ -435,10 +526,11 @@ TEST(clang_parser, bitfields_uneven_fields)
   EXPECT_TRUE(foo->GetField("e").type.IsIntTy());
   EXPECT_EQ(foo->GetField("e").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("e").offset, 3);
-  EXPECT_TRUE(foo->GetField("e").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("e").bitfield.has_value());
   EXPECT_EQ(foo->GetField("e").bitfield->read_bytes, 1U);
   EXPECT_EQ(foo->GetField("e").bitfield->access_rshift, 1U);
   EXPECT_EQ(foo->GetField("e").bitfield->mask, 0x7FU);
+  // NOLINTEND(bugprone-unchecked-optional-access)
 }
 
 TEST(clang_parser, bitfields_with_padding)
@@ -449,6 +541,10 @@ TEST(clang_parser, bitfields_with_padding)
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo"));
   auto foo = bpftrace.structs.Lookup("struct Foo").lock();
 
+  // clang-tidy doesn't seem to acknowledge that ASSERT_*() will
+  // return from function so that these are in fact checked accesses.
+  //
+  // NOLINTBEGIN(bugprone-unchecked-optional-access)
   EXPECT_EQ(foo->size, 16);
   ASSERT_EQ(foo->fields.size(), 4U);
   ASSERT_TRUE(foo->HasField("pad"));
@@ -459,7 +555,7 @@ TEST(clang_parser, bitfields_with_padding)
   EXPECT_TRUE(foo->GetField("a").type.IsIntTy());
   EXPECT_EQ(foo->GetField("a").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("a").offset, 4);
-  EXPECT_TRUE(foo->GetField("a").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("a").bitfield.has_value());
   EXPECT_EQ(foo->GetField("a").bitfield->read_bytes, 4U);
   EXPECT_EQ(foo->GetField("a").bitfield->access_rshift, 0U);
   EXPECT_EQ(foo->GetField("a").bitfield->mask, 0xFFFFFFFU);
@@ -467,10 +563,11 @@ TEST(clang_parser, bitfields_with_padding)
   EXPECT_TRUE(foo->GetField("b").type.IsIntTy());
   EXPECT_EQ(foo->GetField("b").type.GetSize(), 4U);
   EXPECT_EQ(foo->GetField("b").offset, 7);
-  EXPECT_TRUE(foo->GetField("b").bitfield.has_value());
+  ASSERT_TRUE(foo->GetField("b").bitfield.has_value());
   EXPECT_EQ(foo->GetField("b").bitfield->read_bytes, 1U);
   EXPECT_EQ(foo->GetField("b").bitfield->access_rshift, 4U);
   EXPECT_EQ(foo->GetField("b").bitfield->mask, 0xFU);
+  // NOLINTEND(bugprone-unchecked-optional-access)
 }
 
 TEST(clang_parser, builtin_headers)
@@ -797,6 +894,4 @@ struct _tracepoint_irq_irq_handler_entry
   EXPECT_EQ(s->GetField("name").type.GetIntBitWidth(), 64ULL);
 }
 
-} // namespace clang_parser
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::clang_parser
