@@ -10,7 +10,7 @@ namespace codegen {
 
 TEST(codegen, args_multiple_tracepoints_wild)
 {
-  test("tracepoint:sched:sched_* { @[args.common_field] = count(); }", NAME);
+  test("tracepoint:sched:sched_* { @[args.common_field] = 1; }", NAME);
 }
 
 } // namespace codegen
