@@ -11,6 +11,13 @@ TEST(codegen, call_offsetof)
        NAME);
 }
 
+TEST(codegen, call_offsetof_sub_field)
+{
+  test("struct Foo { struct Bar { int a; } d; }"
+       "BEGIN { @x = offsetof(struct Foo, d.a); exit(); }",
+       NAME);
+}
+
 } // namespace codegen
 } // namespace test
 } // namespace bpftrace
