@@ -6,7 +6,7 @@ namespace codegen {
 
 TEST(codegen, cast_int_to_arr)
 {
-  test("kprobe:f { $a=(uint8[8])pid; @ = $a[0]; }", NAME);
+  test("kprobe:f { $a=(uint8[8])0; @ = $a[0]; }", NAME);
 }
 
 } // namespace codegen
