@@ -6,11 +6,10 @@ namespace codegen {
 
 TEST(codegen, fentry_recursion_check)
 {
-  MockBPFtrace bpftrace;
-  bpftrace.need_recursion_check_ = true;
-  bpftrace.feature_ = std::make_unique<MockBPFfeature>(true);
+  auto bpftrace = get_mock_bpftrace();
+  bpftrace->need_recursion_check_ = true;
 
-  test(bpftrace,
+  test(*bpftrace,
        "fentry:queued_spin_lock_slowpath { }"
        "tracepoint:exceptions:page_fault_user { }",
        NAME);
@@ -18,11 +17,10 @@ TEST(codegen, fentry_recursion_check)
 
 TEST(codegen, fentry_recursion_check_with_predicate)
 {
-  MockBPFtrace bpftrace;
-  bpftrace.need_recursion_check_ = true;
-  bpftrace.feature_ = std::make_unique<MockBPFfeature>(true);
+  auto bpftrace = get_mock_bpftrace();
+  bpftrace->need_recursion_check_ = true;
 
-  test(bpftrace, "fentry:queued_spin_lock_slowpath / pid == 1234 / { }", NAME);
+  test(*bpftrace, "fentry:queued_spin_lock_slowpath / pid == 1234 / { }", NAME);
 }
 
 } // namespace codegen
