@@ -6,9 +6,7 @@ namespace codegen {
 
 TEST(codegen, optional_positional_parameter)
 {
-  test("BEGIN { @x = $1; @y = str($2) }",
-
-       NAME);
+  test("BEGIN { @x = $1; @y = str($2) }", NAME);
 }
 
 } // namespace codegen
