@@ -6,10 +6,9 @@ namespace codegen {
 
 TEST(codegen, ptr_to_ptr)
 {
-  test(
-      R"PROG(kprobe:f { $pp = (int32 **)0; printf("%d\n", **kptr($pp)); })PROG",
+  test(R"PROG(kprobe:f { $pp = (int32 **)0; $res = **kptr($pp); })PROG",
 
-      NAME);
+       NAME);
 }
 
 } // namespace codegen
