@@ -9,7 +9,7 @@ TEST(codegen, struct_save_nested)
   test("struct Foo { int m; struct { int x; int y; } bar; int n; }"
        "kprobe:f"
        "{"
-       "  @foo = (struct Foo)0;"
+       "  @foo = *(struct Foo*)arg0;"
        "  @bar = @foo.bar;"
        "  @x = @foo.bar.x;"
        "}",
