@@ -7,9 +7,7 @@
 #include "driver.h"
 #include "mocks.h"
 
-namespace bpftrace {
-namespace test {
-namespace config_analyser {
+namespace bpftrace::test::config_analyser {
 
 using ::testing::_;
 
@@ -26,14 +24,14 @@ void test(BPFtrace &bpftrace,
   ASSERT_EQ(driver.parse_str(input), 0);
 
   ClangParser clang;
-  ASSERT_TRUE(clang.parse(driver.root.get(), bpftrace));
+  ASSERT_TRUE(clang.parse(driver.ctx.root, bpftrace));
 
   ASSERT_EQ(driver.parse_str(input), 0);
   out.str("");
-  ast::SemanticAnalyser semantics(driver.root.get(), bpftrace, out, false);
+  ast::SemanticAnalyser semantics(driver.ctx, bpftrace, out, false);
   ASSERT_EQ(semantics.analyse(), 0) << msg.str() << out.str();
 
-  ast::ConfigAnalyser config_analyser(driver.root.get(), bpftrace, out);
+  ast::ConfigAnalyser config_analyser(driver.ctx, bpftrace, out);
   EXPECT_EQ(config_analyser.analyse(), expected_result)
       << msg.str() << out.str();
   if (expected_error.data()) {
@@ -46,7 +44,7 @@ void test(BPFtrace &bpftrace,
 void test(const std::string &input, bool expected_result)
 {
   auto bpftrace = get_mock_bpftrace();
-  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
   test(*bpftrace, input, {}, expected_result);
 }
 
@@ -55,7 +53,7 @@ void test(const std::string &input,
           bool expected_result)
 {
   auto bpftrace = get_mock_bpftrace();
-  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
   test(*bpftrace, input, expected_error, expected_result);
 }
 
@@ -90,7 +88,7 @@ config = { BAD_CONFIG=1 } BEGIN { }
        false);
   test(
       "config = { BPFTRACE_MAX_PROBES=\"hello\" } BEGIN { }",
-      R"(stdin:1:12-32: ERROR: Invalid type for BPFTRACE_MAX_PROBES. Type: string. Expected Type: integer
+      R"(stdin:1:12-32: ERROR: Invalid type for BPFTRACE_MAX_PROBES. Type: string. Expected Type: int
 config = { BPFTRACE_MAX_PROBES="hello" } BEGIN { }
            ~~~~~~~~~~~~~~~~~~~~
 )",
@@ -127,8 +125,22 @@ TEST(config_analyser, config_setting)
   EXPECT_EQ(bpftrace->config_.get(ConfigKeyUserSymbolCacheType::default_),
             UserSymbolCacheType::per_program);
   EXPECT_EQ(bpftrace->config_.get(ConfigKeyInt::log_size), 150);
+
+  // When liblldb is present, the default config for symbol_source is "dwarf",
+  // otherwise the default is "symbol_table".
+#ifdef HAVE_LIBLLDB
+  EXPECT_EQ(bpftrace->config_.get(ConfigKeySymbolSource::default_),
+            ConfigSymbolSource::dwarf);
+  test(*bpftrace, "config = { symbol_source = \"symbol_table\" } BEGIN { }");
+  EXPECT_EQ(bpftrace->config_.get(ConfigKeySymbolSource::default_),
+            ConfigSymbolSource::symbol_table);
+#else
+  EXPECT_EQ(bpftrace->config_.get(ConfigKeySymbolSource::default_),
+            ConfigSymbolSource::symbol_table);
+  test(*bpftrace, "config = { symbol_source = \"dwarf\" } BEGIN { }");
+  EXPECT_EQ(bpftrace->config_.get(ConfigKeySymbolSource::default_),
+            ConfigSymbolSource::dwarf);
+#endif
 }
 
-} // namespace config_analyser
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::config_analyser
