@@ -4,6 +4,7 @@
 
 #include <cstdio>
 #include <fcntl.h>
+#include <fstream>
 #include <stdexcept>
 
 #include "data/dwarf_data.h"
@@ -32,4 +33,5 @@ protected:
   }
 
   static constexpr const char *bin_ = "/tmp/bpftrace-test-dwarf-data";
+  static constexpr const char *cxx_bin_ = dwarf_data_cxx_path;
 };
