@@ -1,8 +1,7 @@
 #include "mocks.h"
 #include "tracefs.h"
 
-namespace bpftrace {
-namespace test {
+namespace bpftrace::test {
 
 using ::testing::_;
 using ::testing::NiceMock;
@@ -11,27 +10,23 @@ using ::testing::StrictMock;
 
 void setup_mock_probe_matcher(MockProbeMatcher &matcher)
 {
-  ON_CALL(matcher, get_symbols_from_traceable_funcs(false))
-      .WillByDefault([](void) {
-        std::string ksyms = "SyS_read\n"
-                            "sys_read\n"
-                            "sys_write\n"
-                            "my_one\n"
-                            "my_two\n";
-        auto myval = std::unique_ptr<std::istream>(
-            new std::istringstream(ksyms));
-        return myval;
-      });
+  ON_CALL(matcher, get_symbols_from_traceable_funcs(false)).WillByDefault([]() {
+    std::string ksyms = "SyS_read\n"
+                        "sys_read\n"
+                        "sys_write\n"
+                        "my_one\n"
+                        "my_two\n";
+    auto myval = std::unique_ptr<std::istream>(new std::istringstream(ksyms));
+    return myval;
+  });
 
-  ON_CALL(matcher, get_symbols_from_traceable_funcs(true))
-      .WillByDefault([](void) {
-        std::string ksyms = "kernel_mod:func_in_mod\n"
-                            "kernel_mod:other_func_in_mod\n"
-                            "other_kernel_mod:func_in_mod\n";
-        auto myval = std::unique_ptr<std::istream>(
-            new std::istringstream(ksyms));
-        return myval;
-      });
+  ON_CALL(matcher, get_symbols_from_traceable_funcs(true)).WillByDefault([]() {
+    std::string ksyms = "kernel_mod:func_in_mod\n"
+                        "kernel_mod:other_func_in_mod\n"
+                        "other_kernel_mod:func_in_mod\n";
+    auto myval = std::unique_ptr<std::istream>(new std::istringstream(ksyms));
+    return myval;
+  });
 
   ON_CALL(matcher, get_symbols_from_file(tracefs::available_events()))
       .WillByDefault([](const std::string &) {
@@ -86,6 +81,8 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
 
 void setup_mock_bpftrace(MockBPFtrace &bpftrace)
 {
+  bpftrace.delta_taitime_ = timespec{};
+
   bpftrace.parse_btf({ "vmlinux" });
   // Fill in some default tracepoint struct definitions
   bpftrace.structs.Add("struct _tracepoint_sched_sched_one", 8);
@@ -136,6 +133,10 @@ void setup_mock_bpftrace(MockBPFtrace &bpftrace)
   bpftrace.structs.Lookup("struct _tracepoint_btf_tag")
       .lock()
       ->AddField("real_parent", ptr_type_w_bad_tag, 16, std::nullopt, false);
+
+  // Even though this is set to 1 by default, make it 0 here
+  // to reduce the amount of repeated generated IR in the codegen tests
+  bpftrace.helper_check_level_ = 0;
 }
 
 std::unique_ptr<MockBPFtrace> get_mock_bpftrace()
@@ -148,6 +149,8 @@ std::unique_ptr<MockBPFtrace> get_mock_bpftrace()
   setup_mock_probe_matcher(*probe_matcher);
   bpftrace->set_mock_probe_matcher(std::move(probe_matcher));
 
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
   return bpftrace;
 }
 
@@ -161,6 +164,8 @@ std::unique_ptr<MockBPFtrace> get_strict_mock_bpftrace()
   setup_mock_probe_matcher(*probe_matcher);
   bpftrace->set_mock_probe_matcher(std::move(probe_matcher));
 
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
   return bpftrace;
 }
 
@@ -185,5 +190,4 @@ std::unique_ptr<MockUSDTHelper> get_mock_usdt_helper(int num_locations)
   return usdt_helper;
 }
 
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test
