@@ -75,17 +75,17 @@ public:
     return { "mock_vmlinux" };
   }
 
-  const struct stat &get_pidns_self_stat() const override
+  const std::optional<struct stat> &get_pidns_self_stat() const override
   {
-    static const struct stat init_pid_namespace = []() {
-      struct stat s {};
+    static const std::optional<struct stat> init_pid_namespace = []() {
+      struct stat s{};
       s.st_ino = 0xeffffffc; // PROC_PID_INIT_INO
-      return s;
+      return std::optional{ s };
     }();
-    static const struct stat child_pid_namespace = []() {
-      struct stat s {};
+    static const std::optional<struct stat> child_pid_namespace = []() {
+      struct stat s{};
       s.st_ino = 0xf0000011; // Arbitrary user namespace
-      return s;
+      return std::optional{ s };
     }();
 
     if (mock_in_init_pid_ns)
