@@ -4,6 +4,7 @@ from collections import namedtuple
 import os
 import platform
 
+DEFAULT_TIMEOUT = 5
 
 class RequiredFieldError(Exception):
     pass
@@ -32,6 +33,7 @@ TestStruct = namedtuple(
         'timeout',
         'befores',
         'after',
+        'setup',
         'cleanup',
         'suite',
         'kernel_min',
@@ -44,6 +46,7 @@ TestStruct = namedtuple(
         'will_fail',
         'new_pidns',
         'skip_if_env_has',
+        'return_code',
     ],
 )
 
@@ -112,6 +115,7 @@ class TestParser(object):
         timeout = ''
         befores = []
         after = ''
+        setup = ''
         cleanup = ''
         kernel_min = ''
         kernel_max = ''
@@ -123,6 +127,7 @@ class TestParser(object):
         will_fail = False
         new_pidns = False
         skip_if_env_has = None
+        return_code = None
         prev_item_name = ''
 
         for item in test:
@@ -135,13 +140,13 @@ class TestParser(object):
                 if prev_item_name == 'PROG':
                     prog += '\n' + line
                     continue
-                elif prev_item_name == 'EXPECT':
+                elif prev_item_name in ('EXPECT', 'EXPECT_REGEX'):
                     expects[-1].expect += '\n' + line
                     continue
 
-            item_split = item.split()
+            item_split = item.strip().split(maxsplit=1)
             item_name = item_split[0]
-            line = ' '.join(item_split[1:])
+            line = item_split[1] if len(item_split) > 1 else ""
             prev_item_name = item_name
 
             if item_name == 'NAME':
@@ -170,6 +175,8 @@ class TestParser(object):
                 befores.append(line)
             elif item_name == 'AFTER':
                 after = line
+            elif item_name == 'SETUP':
+                setup = line
             elif item_name == 'CLEANUP':
                 cleanup = line
             elif item_name == 'MIN_KERNEL':
@@ -188,7 +195,7 @@ class TestParser(object):
                 features = {
                     "loop",
                     "btf",
-                    "kfunc",
+                    "fentry",
                     "probe_read_kernel",
                     "dpath",
                     "uprobe_refcount",
@@ -196,6 +203,7 @@ class TestParser(object):
                     "iter",
                     "libpath_resolv",
                     "dwarf",
+                    "kernel_dwarf",
                     "aot",
                     "kprobe_multi",
                     "uprobe_multi",
@@ -222,6 +230,8 @@ class TestParser(object):
             elif item_name == "SKIP_IF_ENV_HAS":
                 parts = line.split("=")
                 skip_if_env_has = (parts[0], parts[1])
+            elif item_name == "RETURN_CODE":
+                return_code = int(line.strip(' '))
             else:
                 raise UnknownFieldError('Field %s is unknown. Suite: %s' % (item_name, test_suite))
 
@@ -231,12 +241,15 @@ class TestParser(object):
             raise RequiredFieldError('Test RUN or PROG is required. Suite: ' + test_suite)
         elif run != '' and prog != '':
             raise InvalidFieldError('Test RUN and PROG both specified. Suit: ' + test_suite)
-        elif len(expects) == 0:
-            raise RequiredFieldError('At leat one test EXPECT (or variation) is required. Suite: ' + test_suite)
+        elif len(expects) == 0 and return_code is None:
+            raise RequiredFieldError('At least one test EXPECT (or variation) is required. Suite: ' + test_suite)
         elif len(expects) > 1 and has_exact_expect:
             raise InvalidFieldError('EXPECT_JSON or EXPECT_FILE can not be used with other EXPECTs. Suite: ' + test_suite)
         elif timeout == '':
-            raise RequiredFieldError('Test TIMEOUT is required. Suite: ' + test_suite)
+            timeout = DEFAULT_TIMEOUT
+
+        if return_code is None:
+            return_code = 0
 
         return TestStruct(
             name,
@@ -247,6 +260,7 @@ class TestParser(object):
             timeout,
             befores,
             after,
+            setup,
             cleanup,
             test_suite,
             kernel_min,
@@ -259,4 +273,5 @@ class TestParser(object):
             will_fail,
             new_pidns,
             skip_if_env_has,
+            return_code,
         )
