@@ -3,8 +3,7 @@
 
 int main()
 {
-  while (1)
-  {
+  while (1) {
     myclock();
   }
   return 0;
