@@ -5,19 +5,23 @@
 #else
 #define DTRACE_PROBE2(a, b, c, d) (void)0
 #endif
+#include <stdio.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <stdio.h>
 
-__extension__ unsigned short tracetest_testprobe_semaphore __attribute__ ((unused)) __attribute__ ((section (".probes"))) __attribute__ ((visibility ("hidden")));
+__extension__ unsigned short tracetest_testprobe_semaphore
+    __attribute__((unused)) __attribute__((section(".probes")))
+    __attribute__((visibility("hidden")));
 
-static long
-myclock() {
+static long myclock()
+{
   char buffer[100];
   struct timeval tv;
   gettimeofday(&tv, NULL);
-  sprintf(buffer, "tracetest_testprobe_semaphore: %d\n", tracetest_testprobe_semaphore);
-  DTRACE_PROBE2(tracetest,  testprobe,  tv.tv_sec, buffer);
+  sprintf(buffer,
+          "tracetest_testprobe_semaphore: %d\n",
+          tracetest_testprobe_semaphore);
+  DTRACE_PROBE2(tracetest, testprobe, tv.tv_sec, buffer);
   return tv.tv_sec;
 }
 
